/************************************************************\
*  LCD kontroller V1.0                                       *
*                                                            *
*                                                            *
*                                                            *
*                                                            *
*                                                            *
*                                                            *
*                                                            *
*                                                            *
*                                                            *
*                                                            *
*                                                            *
*                                                            *
*                                                            *
*                                                            *
*  LCD kijelzohoz       |      RAM  csatlakozas              *
* **********************|*************************************
*  P3   LCD_DISP_OFF    |             P66  RAM_DATA0         *
*  P96  LCD_FRAME       |             P65  RAM_DATA1         *
*  P95  LCD_CP          |             P62  RAM_DATA2         *
*  P93  LCD_LOAD        |             P60  RAM_DATA3         *
*  P91  CLK_100MHz      |             P43  RAM_DATA4         *
*  P97  LCD_D3          |             P41  RAM_DATA5         * 
*  P87  LCD_D2          |             P54  RAM_DATA6         *
*  P86  LCD_D1          |             P59  RAM_DATA7   	    *
*  P84  LCD_D0          |                                    *
*                       |       		  P72  nRAM_OE           *
*                       |             P58  nRAM_WE           *
*_______________________/             P68  nRAM_CE0          *
*  	                                                       *
*  P73  RAM_ADDRESS0                  P46  RAM_ADDRESS9      *
*  P71  RAM_ADDRESS1                  P44  RAM_ADDRESS10     *
*  P70  RAM_ADDRESS2                  P56  RAM_ADDRESS11     *
*  P69  RAM_ADDRESS3                  P52  RAM_ADDRESS12     *
*  P57  RAM_ADDRESS4                  P74  RAM_ADDRESS13     *
*  P55  RAM_ADDRESS5                  P80  RAM_ADDRESS14     *
*  P53  RAM_ADDRESS6                  P81  RAM_ADDRESS15     *
*  P47  RAM_ADDRESS7                  P82  RAM_ADDRESS16     *
*  P45  RAM_ADDRESS8                                         *
* ___________________________________________________________*
*                   |                                        *
*  CPU csatlakozas  |                                        *
* *******************                                        *
*                                                            *
*  P4   nCPU_RD                       P6   CPU_ADDRESS0		 *
*	P5   nCPU_WR  	                    P40  CPU_ADDRESS1      *
*  P6                                 P7   CPU_ADDRESS2      *
*  P7                                                        *
*  P8                                                        *
*                                                            *
*  P20  CPU_DATA0                                            *
*  P21  CPU_DATA1                                            *
*  P22  CPU_DATA2                                            *
*  P30  CPU_DATA3                                            *
*  P31  CPU_DATA4                                            *
*  P32  CPU_DATA5                                            *
*  P34  CPU_DATA6                                            *
*  P83  CPU_DATA7                                            *
*                                                            *
\************************************************************/


module LCD(
	CLK_100MHz,
	LCD_DATA,
	LCD_FRAME,
	LCD_LOAD,
	LCD_CP,
	nLCD_DISP_OFF,
	RAM_ADDRESS,
	RAM_DATA,
	nRAM_CE0,
	nRAM_OE,
	nRAM_WE,	
	nCPU_RD,
	nCPU_WR,
	CPU_DATA,
	CPU_ADDRESS,
	PROBA
);

output PROBA;
reg PROBA;

// RAM kivezetesekhez tartozo delkaraciok
//
output [16:0]	RAM_ADDRESS;		// RAM cim
reg	 [16:0]	RAM_ADDRESS;
inout	 [7:0] 	RAM_DATA;			// RAM adat
output 			nRAM_CE0;			// RAM CE0
reg				nRAM_CE0 = 1;
output			nRAM_OE;	    		// RAM OE
reg				nRAM_OE  = 1;
output			nRAM_WE;				// RAM WE
reg 				nRAM_WE  = 1;


// LCD kivezetesekhez tartozo deklaraciok
//
output [3:0]   LCD_DATA; 	      // LCD 4 bit adat
reg    [3:0]   LCD_DATA = 0;
output			LCD_FRAME;	      // LCD V sync.
reg				LCD_FRAME = 0;
output			LCD_LOAD;		   // LCD H sync
reg				LCD_LOAD = 0;
output			LCD_CP;		      // LCD clock
reg				LCD_CP = 0;
output			nLCD_DISP_OFF;		// LCD engedelyezes=1, tiltas=0
reg				nLCD_DISP_OFF=0;

// CPU kivezetesekhez tartozo deklaraciok
//
input				nCPU_RD;	 			// CPU RD jel aktiv "L" szint
input				nCPU_WR;				// CPU WR jel aktiv "L" szint
input	[2:0]		CPU_ADDRESS;		// CPU felol A0, A1, A2 cimvezetek
inout	[7:0]		CPU_DATA;			// CPU felol 8bit-es adatbusz


// Orajelbemenethez es eloosztokhoz tartozo deklaraciok
//
input			   CLK_100MHz;	      // 100MHz orajel
reg				clk_div_xx;			// 50MHz-re osztja a 100MHz-et.
reg   [6:0]		cp_clk_div = 0;	// Elooszto a shift clock jelhez

// Eldetektalashoz tartozo deklaraciok
//
reg	[1:0]		lcd_cp_edge  	=  0;  	// LCD CP jel eldetektalasahoz mintaveteli shift regiszter
reg	[1:0]		ncpu_rd_edge 	=  0;		// CPU RD jel eldetektalasahoz mintaveteli shift regiszter
reg	[1:0]		ncpu_wr_edge 	=  0;		// CPU WR jel eldetektalasahoz mintaveteli shift regiszter

// State machine deklaraciok
//
reg	[4:0]		ram_cikl			   = 0;	// 0 nem hasznalja senki a RAM-ot
													// 1..3 CPU ir a video RAM-ba
													// 4..6 ????
													// 7..9   layer 1 olvasasa a video RAM-bol
													// 10..12 layer 2 olvasasa a video RAM-bol
													// 13..15 layer 3 olvasasa a video RAM-bol
													// 16..18 layer 4 olvasasa a video RAM-bol
													// 19..21 layer 5 olvasasa a video RAM-bol
													// 22..24 layer 5 olvasasa a video RAM-bol

// Egyeb belso regiszterek
//
reg	[7:0]		command			 = 0;		// Parancs regiszter
reg	[7:0]		status			 = 0;		// Statusz regiszter
reg	[7:0]		ram_write_data;			// RAM-ba irando adat
reg	[7:0]		cpu_read_data;				// CPU_ADDRESS = 0 - RAM data --> CPU data
reg	[7:0]		cpu_write_data;			// CPU_ADDRESS = 0 - CPU data <-- RAM data
reg	[7:0]		cpu_A7_A0       = 0;		// CPU_ADDRESS = 1 - CPU data --> RAM A7...A0
reg	[7:0]		cpu_A15_A8		 = 0;		// CPU_ADDRESS = 2 - CPU data --> RAM A15...A8
reg				cpu_A16			 = 0;		// CPU_ADDRESS = 3 - CPU data --> RAM A16
reg	[7:0]		layer_grayscale = 255;	// CPU_ADDRESS = 4 - CPU data --> layer_grayscale
													// Az egyes layer-ek fenyerejenek erteket tartalmazza
													// b7 b6 b5 b4 b3 b2 b1 b0
													//  ---	 ---	 ---	 ---
													//	  |     |     |	  |
													//	  |     |	  |		------->	Layer 1
													//   |	  |		------------->	Layer 2
													//   |	   ------------------->	Layer 3
													//    ------------------------->	Layer 4
													//
													// 2 bit/layer 
													//		0 = 1 vilagos 3 sotet
													//		1 = 2 vilagos 2 sotet
													//		2 = 3 vilagos 1 sotet
													//		3 = 4 vilagos 0 sotet

reg	[7:0]		lcd_layer_data_puffer = 0;		// Ebbe mixeli ossze a kulonbozo layer-ek adatait

// Ram hozzaferest kero jelzobitek
//
//reg		cpu_ram_write_req = 0;	// 1 ha a CPU akarja irni a RAM-ot
//reg		cpu_ram_read_req	= 0;  // 1 ha a CPU akarja olvasni a RAM-ot
//reg		layer1_mem_req	   = 0;	// A display terito 1-be irja, ha olvasni akarja a layer 1-et a ram-bol 
//reg		layer2_mem_req	   = 0;	// A display terito 1-be irja, ha olvasni akarja a layer 2-ot a ram-bol 
//reg		layer3_mem_req	   = 0;	// A display terito 1-be irja, ha olvasni akarja a layer 3-at a ram-bol 
//reg		layer4_mem_req	   = 0;	// A display terito 1-be irja, ha olvasni akarja a layer 4-et a ram-bol 
//reg		layer5_mem_req    = 0;	// A display terito 1-be irja, ha olvasni akarja a layer 5-ot a ram-bol,
//reg		layer6_mem_req    = 0;	// A display terito 1-be irja, ha olvasni akarja a layer 6-ot a ram-bol,
											// de ez csak a masodik byte felolvasasa utan torlodik !

// Layer engedelyezo bitek
//			
// A layer_enable regisztert a CPU barmikor 
// valtoztathatja, de a szamozott engedelyezo
// bitek csak minden kepvaltaskor modosulnak.
//
reg	[6:0]	layer_enable 	 = 0;	   // Layer engedelyezo regiszter
reg			layer1_enable	 = 0;		// Layer 1 engedelyezo regiszter 0=disable, 1=enable
reg			layer2_enable	 = 0;		// Layer 2 engedelyezo regiszter 0=disable, 1=enable
reg			layer3_enable	 = 0;		// Layer 3 engedelyezo regiszter 0=disable, 1=enable
reg			layer4_enable	 = 0;		// Layer 4 engedelyezo regiszter 0=disable, 1=enable
reg			layer5_enable	 = 0;		// Layer 5 engedelyezo regiszter 0=disable, 1=enable
reg			layer6_enable	 = 0;		// Layer 6 engedelyezo regiszter 0=disable, 1=enable


// Szamlalokhoz tartozo deklaraciok 
//
reg	[16:0]	display_addr	 = 0;			// A display terites soran az 1 bit/pixel-es kepterites relativ RAM cimet 
														// tartalmazza ami minden masodik LCD shift clock utan novel. ( 0...9599 )
														// Azert minden masodikra mert az video RAM 8 bit-es, de az LCD adatbusz 
														// csak 4bit-es es igy egy RAM olvasara 2 LCD buszciklus jut.

reg	[16:0]	grayscale_display_addr = 0;// A display terites soram a 2 bit/pixel-es kepterites relativ RAM cimet 
														// tartalmazza amit minden LCD shift clock utan novel. ( 0...19199 )


reg   [6:0]		lcd_x_ct = 0;				// LCD oszlop szamlalo 0..78  ( 80 x 4 bit 320 pixel )
reg	[7:0]		lcd_y_ct = 0;				// LCD sor szamlalo    0..239 ( 240 sor )




// Constans deklaraciok
//
parameter Felfuto_el = 1; // a 2bit-es shift regiszterek tartalma 01 ha felfuto el volt
parameter Lefuto_el  = 2; // a 2bit-es shift regiszterek tartalma 10 ha felfuto el volt

parameter		layer1_start	=0;
parameter		layer2_start	=9600;
parameter		layer3_start	=19200;
parameter		layer4_start	=28800;
parameter		layer5_start	=38400;

reg	[1:0]		grayscale_byte_ct = 0;			// A 2bit/pixel-es RAM olvasasnal a felolvasott byte-ot szamolja 			
reg	[1:0]		frame_ct 			= 0;			// minden 240. sor teritese utan novekszik
															// "ROM" tablazat cimenek offsetelesere hasznalom igy megkavarja
															// a kijelzest 4 egymast kiveto sorban es 4 egymas utan kovetkezo
															// kep utan. 

reg	[3:0]		look_up_table_addr;				// A fenyero "ROM" tablazat cime
wire	[7:0]		look_up_table_data;				// A fenyero "ROM" tablazat kimeno adat

reg	[3:0]		rom0_addr;
wire				rom0_data;
reg	[3:0]		rom1_addr;
wire				rom1_data;
reg	[3:0]		rom2_addr;
wire				rom2_data;
reg	[3:0]		rom3_addr;
wire				rom3_data;
reg	[3:0]		rom4_addr;
wire				rom4_data;
reg	[3:0]		rom5_addr;
wire				rom5_data;
reg	[3:0]		rom6_addr;
wire				rom6_data;
reg	[3:0]		rom7_addr;
wire				rom7_data;



look_up_table TABLA1( .rom_addr( look_up_table_addr ), .x( frame_ct[1:0] ), .rom_data( look_up_table_data ) );

rom0 TABLA2( .rom_addr( rom0_addr ), .x( frame_ct[1:0] ), .rom_data( rom0_data ) );
rom1 TABLA3( .rom_addr( rom1_addr ), .x( frame_ct[1:0] ), .rom_data( rom1_data ) );
rom2 TABLA4( .rom_addr( rom2_addr ), .x( frame_ct[1:0] ), .rom_data( rom2_data ) );
rom3 TABLA5( .rom_addr( rom3_addr ), .x( frame_ct[1:0] ), .rom_data( rom3_data ) );
//rom4 TABLA6( .rom_addr( rom4_addr ), .x( frame_ct[1:0] ), .rom_data( rom4_data ) );
//rom5 TABLA7( .rom_addr( rom5_addr ), .x( frame_ct[1:0] ), .rom_data( rom5_data ) );
//rom6 TABLA8( .rom_addr( rom6_addr ), .x( frame_ct[1:0] ), .rom_data( rom6_data ) );
//rom7 TABLA9( .rom_addr( rom7_addr ), .x( frame_ct[1:0] ), .rom_data( rom7_data ) );


reg	[7:0]	mem_req;
wire	[4:0]	mem_req_start;

mem_prioritas_encoder PRIENC1( .input_memreq( mem_req ), .out( mem_req_start ) );

/********************************************************************\
* 	50MHz - es orajel eloallitasa a 100 MHz - bol.                   *
\********************************************************************/
// Global clock buffer azert hogy tudja a fordito hogy ez orajel
BUFG clk_xx ( .I( clk_div_xx ) , .O( CLK_50MHz ) );
always @ ( posedge CLK_100MHz ) begin
	// 100MHz/2
	clk_div_xx <= clk_div_xx + 1;
end

///assign CLK_50MHz = CLK_100MHz;
	
// RAM adatbusz meghajtasa a ram_write_data regiszterbol 
// ha a WE es a CE0 aktiv "L" szintu kulonben (TS)
// Minden ram-ba irando adatot ebbe a regiszterbe kell tolteni.
assign RAM_DATA = ( !nRAM_WE && !nRAM_CE0 ) ? ram_write_data : 8'hzz;

//------------------------------------------------------------------
// A belso regiszterek kikuldese CPU fele meno adatbuszra ha az RD jel 
// aktiv "L" szintu es az WR jel "H" szintu kulonbenm (TS)
// A2..A0 CPU_ADDRESS-tol fuggoen kozvetlenul 7 belso regiszter erheto el.
//
// addr 0	RAM adat 						 ----> CPU
// addr 1   
// addr 2   
// addr 3   
//	addr 4 	Parancs regiszter. Ettol fuggoen a 0-as cimrol olvasott adatok
//				a CPU adatbuszra kerulnek az alabbi regiszterekbol
//
//				command = 0	<--- cpu_read_data
//
//------------------------------------------------------------------
assign CPU_DATA = ( nCPU_RD == 0 && nCPU_WR == 1 && CPU_ADDRESS == 0 ) ? cpu_read_data : 
						( nCPU_RD == 0 && nCPU_WR == 1 && CPU_ADDRESS == 3 ) ? { 0, nLCD_DISP_OFF, layer_enable[5:0] }:
						( nCPU_RD == 0 && nCPU_WR == 1 && CPU_ADDRESS == 4 ) ? layer_grayscale: 8'hzz; 


//------------------------------------------------------------------
// CPU felol jovo adat betoltese a belso regiszterekbe a WR jel
// lefuto ele utan ha az RD jel "H" sizntu
// A2..A0 CPU_ADDRESS-tol fuggoen kozvetlenul 7 belso regiszter erheto el.
// addr 0	CPU ---> adatregiszterek
// addr 1   CPU ---> RAM cim also  byte  A7...A0
// addr 2   CPU ---> RAM cim felso byte A15...A8
// addr 3   CPU ---> layer engedelyezo bitek ( 1 engedelyezes, 0 tiltas. )
//
// 	b7 b6 b5 b4 b3 b2 b1 b0 --->  layer_enable regiszter
//	  	 |  |  |  |  |  |  |  |
//	  	 |  |  |	 |  |  |  |   ---->	layer_enable[0] Layer 1 ( 1bit/pixel )
//     |	 |  |  |  |  |   ------->	layer_enable[1] Layer 2 ( 1bit/pixel	)
//     |	 |  |  |	 |   ---------->	layer_enable[2] Layer 3 ( 1bit/pixel	)
//     |  |  |  |   ------------->  layer_enable[3] Layer 4 ( 1bit/pixel	)
//     |  |  |   ---------------->	layer_enable[4] Layer 5 ( 2bit/pixel	)
//		 |	 |   ------------------->  layer_enable[5] Layer 6 ( karakteres	)
//		 |	  ---------------------->  Display 0 = tiltas, 1 = engedelyezes
//		  ------------------------->	RAM cim A16
//
//	addr 4 	Parancs regiszter. Ettol fuggoen a 0-as cimre irt adatok
//				az alabbi regiszterekbe kerulnek.
//
//				command = 0	-- > cpu_write_data
//				command = 1 ---> layer_grayscale
//
// 									layer 1...4 fenyerobeallitasok 
//										( 0 ertek a legkissebb a 3-as legnagyobb fenyero )
//
//										b7 b6 b5 b4 b3 b2 b1 b0  -->	layer_grayscale regiszter
//									    ---   ---   ---   ---
//									  	  |     |     |     | 
//	  									  |     |     |     | 
//									     |     |     |      ------>	Layer 1
//      								  |	  |      ------------>	Layer 2
// 								     |      ------------------>  Layer 3
//									      ------------------------>	Layer 4
//
//------------------------------------------------------------------
						 
always @ ( posedge CLK_50MHz ) begin

	// RAM olvasas kesz
	PROBA = mem_req[6];

	// CPU RAM olvasas jelzes torlese ha mar belekezdett
	if ( ram_cikl == 2 ) mem_req[0] <= 0; 
	// CPU RAM iras jelzes torlese ha mar belekezdett
	if ( ram_cikl == 5 ) mem_req[1] <= 0; 

	if ( nCPU_WR == 1 && ncpu_rd_edge == Lefuto_el ) 
		begin
			if ( CPU_ADDRESS == 0 ) 
				begin
					case ( command )
						0: // CPU RAM olvasas
							begin
								mem_req[0] <= 1;		 // RAM olvasas hozzaferes igeny jelzes irashoz
							end
					endcase
				end
		end

	

	
	// Ha az nCPU_RD nem aktiv, akkor vizsgalja hogy akar-e irni a CPU
	if ( nCPU_RD == 1 && ncpu_wr_edge == Lefuto_el ) 
		begin	

			if ( CPU_ADDRESS == 0 ) 
				begin 
					case ( command )
						0:	// CPU RAM iras
							begin 
								cpu_write_data <= CPU_DATA; // CPU adat a cpu_write_data regiszterbe
								mem_req[1] <= 1;				 // RAM irass hozzaferes igeny jelzes irashoz
							end
						1:	// Layer1..4 fenyero regiszter irasa
							layer_grayscale   <= CPU_DATA;
					endcase
				end

			//	RAM cimzo regiszter also byte irasa
			if ( CPU_ADDRESS == 1 ) cpu_A7_A0         <= CPU_DATA;
	
			//	RAM cimzo regiszter felso byte irasa
			if ( CPU_ADDRESS == 2 ) cpu_A15_A8        <= CPU_DATA;

			if ( CPU_ADDRESS == 3 )
				begin 
					cpu_A16 				  <= CPU_DATA[7];		// CPU RAM A16 iras
					layer_enable[6:0]   <= CPU_DATA[6:0];	// Display es Layer engedelyezo/tilto regiszter irasa
				end
	
			// command regiszter irasa
			if ( CPU_ADDRESS == 4 ) command <= CPU_DATA;
		end
end

/********************************************************************\
* 	Eldetektalasok                                                    *
* 	                                                                  *
* 	00 = "L" szint, 01 = Felfuto el, 11 = "H" szint, 10 = Lefuto el   *
* 	                                                                  *
\********************************************************************/
always @ ( posedge CLK_50MHz ) begin

	// LCD shift clock jele az eldetektalashoz egy 2bit-es shiftregiszterbe
	lcd_cp_edge[0] <= LCD_CP;
	lcd_cp_edge[1] <= lcd_cp_edge[0];

	// CPU RD jel az eldetektalashoz egy 2bit-es shiftregiszterbe
	ncpu_rd_edge[0] <= nCPU_RD;
	ncpu_rd_edge[1] <= ncpu_rd_edge[0];

	// CPU WR jel az eldetektalashoz a 2bit-es shiftregiszterbe
	ncpu_wr_edge[0] <= nCPU_WR;
	ncpu_wr_edge[1] <= ncpu_wr_edge[0];

end
//------------------------------------------------------------------


always @ ( posedge CLK_50MHz ) begin

	if ( lcd_cp_edge == Felfuto_el && lcd_x_ct == 79 && lcd_y_ct == 239 ) begin 
		// Layer engedelyezesek layer_enable regiszter bitjeitol fuggoen
		layer1_enable = layer_enable[0];
		layer2_enable = layer_enable[1];
		layer3_enable = layer_enable[2];
		layer4_enable = layer_enable[3];
		layer5_enable = layer_enable[4];
		// Minden kepvaltas alkalmaval novekszik. A fenyerobeallitasoknal van hasznalva.
		frame_ct <= frame_ct + 1;
	end


	// Az LCD shift clock jel felfuto elenel minden engedelyezett layer-hez 
	// tartozo memoria hozzaferest kero bit-et 1-be irja.
	if ( lcd_cp_edge == Felfuto_el) begin 
		if ( lcd_x_ct[0] == 1 ) begin
			if ( layer1_enable == 1 ) mem_req[2] <= 1;	// Layer 1 RAM memoria olvasas keres jelzes
			if ( layer2_enable == 1 ) mem_req[3] <= 1;	// Layer 2 RAM memoria olvasas keres jelzes
			if ( layer3_enable == 1 ) mem_req[4] <= 1;	// Layer 3 RAM memoria olvasas keres jelzes
			if ( layer4_enable == 1 ) mem_req[5] <= 1;	// Layer 4 RAM memoria olvasas keres jelzes
			if ( layer5_enable == 1 ) mem_req[6] <= 1;	// Layer 5 RAM memoria olvasas keres jelzes
		end
	end	

	// Prioritasba rendezve a ram hozzaferes engedelyezese
	// Sorrendben legmagasabb 	: 	CPU olvasas
	//										CPU iras
	//										Layer 1
	//										Layer 2
	//										Layer 3
	//										Layer 4
	//										Layer 5
	//			  Legalacsonyabb	:											

	if ( ram_cikl == 0 ) ram_cikl <= mem_req_start;

	//if ( cpu_ram_read_req == 1 && ram_cikl == 0 ) ram_cikl <= 1;
	//else if ( cpu_ram_write_req == 1 && ram_cikl == 0 ) ram_cikl <= 4;
	//else if ( layer1_mem_req == 1 && ram_cikl == 0 ) ram_cikl 	 <= 7;
	//else if ( layer2_mem_req == 1 && ram_cikl == 0 ) ram_cikl    <= 10;
	//else if ( layer3_mem_req == 1 && ram_cikl == 0 ) ram_cikl    <= 13;
	//else if ( layer4_mem_req == 1 && ram_cikl == 0 ) ram_cikl    <= 16;
	//else if ( layer5_mem_req == 1 && ram_cikl == 0 ) ram_cikl    <= 19;
	
			
	
	case ( ram_cikl )
		1: // CPU olvas a RAM-bol
			begin
				RAM_ADDRESS 	<= { cpu_A16, cpu_A15_A8[7:0], cpu_A7_A0[7:0] }; 
				nRAM_CE0	<= 0;
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				ram_cikl <= 2;	
			end
		2:
			begin
				ram_cikl <= 3;
			end		
		3:
			begin
				cpu_read_data = RAM_DATA;
				nRAM_CE0 <= 1;
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				//ram_cikl <= 0;
				ram_cikl <= mem_req_start;
			end

		4:	// CPU ir a video RAM-ba
			begin
				RAM_ADDRESS 	<= { cpu_A16, cpu_A15_A8[7:0], cpu_A7_A0[7:0] }; 
				ram_write_data <= cpu_write_data;
				nRAM_CE0	<= 0;
				nRAM_WE	<= 1;
				nRAM_OE  <= 1;
				ram_cikl <= 5;	
			end
		5:
			begin
				nRAM_CE0	<= 0;
				nRAM_WE	<= 0;
				nRAM_OE  <= 1;
				ram_cikl <= 6;	
			end
		6:
			begin
				nRAM_CE0	<= 1;
				nRAM_WE	<= 1;
				nRAM_OE  <= 1;
				ram_cikl <= mem_req_start;
				//ram_cikl <= 0;	
			end
		7:	// A display terito olvassa a layer1-et a video RAM-bol
			begin
				RAM_ADDRESS <= display_addr + layer1_start; 
				nRAM_CE0	<= 0;
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				ram_cikl <= 8;	
				look_up_table_addr = { layer_grayscale[1:0], lcd_y_ct[0], lcd_x_ct[0] };
			end
		8:	
			begin
				ram_cikl <= 9;	
			end
		9:	
			begin
				lcd_layer_data_puffer <= lcd_layer_data_puffer | ( RAM_DATA & look_up_table_data );
				nRAM_CE0 <= 1;
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				mem_req[2] <= 0;
				ram_cikl <= mem_req_start;
				//ram_cikl <= 0;
			end
		10:	// A display terito olvassa a layer2-ot a video RAM-bol
			begin
				RAM_ADDRESS <= display_addr + layer2_start; 
				nRAM_CE0	<= 0;
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				look_up_table_addr = { layer_grayscale[3:2], lcd_y_ct[0], lcd_x_ct[0] };
				ram_cikl <= 11;	
			end
		11:	
			begin
				ram_cikl <= 12;	
			end
		12:	
			begin
				lcd_layer_data_puffer <= lcd_layer_data_puffer | ( RAM_DATA & look_up_table_data );
				nRAM_CE0 <= 1;
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				mem_req[3] <= 0;
				ram_cikl <= mem_req_start;
				//ram_cikl <= 0;
			end
		13:	// A display terito olvassa a layer3-at a video RAM-bol
			begin
				RAM_ADDRESS <= display_addr + layer3_start; 
				nRAM_CE0	<= 0;
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				look_up_table_addr = { layer_grayscale[5:4], lcd_y_ct[0], lcd_x_ct[0] };
				ram_cikl <= 14;	
			end
		14:	
			begin
				ram_cikl <= 15;	
			end
		15:	
			begin
				lcd_layer_data_puffer <= lcd_layer_data_puffer | ( RAM_DATA & look_up_table_data );
				nRAM_CE0 <= 1;
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				mem_req[4] <= 0;
				ram_cikl <= mem_req_start;
				//ram_cikl <= 0;
			end
		16:	// A display terito olvassa a layer4-et a video RAM-bol
			begin
				RAM_ADDRESS <= display_addr + layer4_start; 
				nRAM_CE0	<= 0;
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				look_up_table_addr = { layer_grayscale[7:6], lcd_y_ct[0], lcd_x_ct[0] };
				ram_cikl <= 17;	
			end
		17:	
			begin
				ram_cikl <= 18;	
			end
		18:	
			begin
				lcd_layer_data_puffer <= lcd_layer_data_puffer | ( RAM_DATA & look_up_table_data );
				nRAM_CE0 <= 1;
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				mem_req[5] <= 0;
				ram_cikl <= mem_req_start;
				//ram_cikl <= 0;
			end
		19:	// A display terito olvassa a layer5-ot a video RAM-bol
			begin
				if ( grayscale_byte_ct == 0 ) 
					RAM_ADDRESS <= grayscale_display_addr + layer5_start; 
				else
					RAM_ADDRESS <= grayscale_display_addr + layer5_start + 1; 

				nRAM_CE0	<= 0;
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				ram_cikl <= 20;	
			end
		20:
			begin
				ram_cikl <= 21;	
			end
		21:
			begin
				rom3_addr = { RAM_DATA[7:6], lcd_y_ct[0], lcd_x_ct[0] };
				rom2_addr = { RAM_DATA[5:4], lcd_y_ct[0], lcd_x_ct[0] };
				rom1_addr = { RAM_DATA[3:2], lcd_y_ct[0], lcd_x_ct[0] };
				rom0_addr = { RAM_DATA[1:0], lcd_y_ct[0], lcd_x_ct[0] };

				if ( grayscale_byte_ct == 0 )
					begin
						//rom7_addr = { RAM_DATA[7:6], lcd_y_ct[0], lcd_x_ct[0] };
						//rom6_addr = { RAM_DATA[5:4], lcd_y_ct[0], lcd_x_ct[0] };
						//rom5_addr = { RAM_DATA[3:2], lcd_y_ct[0], lcd_x_ct[0] };
						//rom4_addr = { RAM_DATA[1:0], lcd_y_ct[0], lcd_x_ct[0] };
						//lcd_layer_data_puffer[7:4] <= lcd_layer_data_puffer[7:4] | { rom7_data, rom6_data, rom5_data, rom4_data };
						lcd_layer_data_puffer[7:4] <= lcd_layer_data_puffer[7:4] | { rom3_data, rom2_data, rom1_data, rom0_data };
						grayscale_byte_ct <= 1;

					end

				if ( grayscale_byte_ct == 1 )
					begin 
						//rom3_addr = { RAM_DATA[7:6], lcd_y_ct[0], lcd_x_ct[0] };
						//rom2_addr = { RAM_DATA[5:4], lcd_y_ct[0], lcd_x_ct[0] };
						//rom1_addr = { RAM_DATA[3:2], lcd_y_ct[0], lcd_x_ct[0] };
						//rom0_addr = { RAM_DATA[1:0], lcd_y_ct[0], lcd_x_ct[0] };
						lcd_layer_data_puffer[3:0] <= lcd_layer_data_puffer[3:0] | { rom3_data, rom2_data, rom1_data, rom0_data };
						grayscale_byte_ct <= 2;
					end
				nRAM_CE0 <= 1;
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				// Csak akkor torli a memoria hozzaferest kero bit-et, ha a masodik byte-ot is felolvasta!
				if ( grayscale_byte_ct == 2 || grayscale_byte_ct == 3 ) 
					begin 
						mem_req[6] <= 0;
						grayscale_byte_ct <= 0;
					end
				ram_cikl <= mem_req_start;
				//ram_cikl <= 0;
			end
	endcase

	// LCD DATA meghajtasa
	// Az LCD shift clock jel felfuto elenel az osszes beolvasott layert a megfelelo 
	// logikai kapcsolattal kiirja az LCD adatbuszra. Ha valamit nem sikerult idoben 
	// kiolvasni a video ram-bol, akkor annak a helyere 0 ertek kerul.
	// Ez akkor fordulhat elo ha tul gyorsan irja vagy olvassa a CPU a video RAM-ot.
	// A 4bit-es LCD busz miatt ket reszletben kuldi ki a RAM-bol kiolvasott adatokat
	// az lcd_x_ct 0. bitjetol fuggoen. Ha 0 akkor a felso 4 bit, ha 1 akkor az also 4 bit.
	if ( lcd_cp_edge == Felfuto_el) begin 
		if ( lcd_x_ct[0] == 0 ) 
			begin 
				LCD_DATA[3:0] <= lcd_layer_data_puffer[7:4]; 
				lcd_layer_data_puffer[7:4] <= 0;
			end 
		else 
			begin 
				LCD_DATA[3:0] <= lcd_layer_data_puffer[3:0]; 
				lcd_layer_data_puffer[3:0] <= 0;
			end
		end	  
	
end





/********************************************************************\
* 													                              *
* 	               LCD szinkron jelek eloallitasa       				   *	
* 													                              *
\********************************************************************/
/********************************************************************\
* 	 LCD_CP    *                                       				   *	
***************     >|       |< 740ns					                  *
* 		                ___     ___     _             ___	   __       *
*         	 CP     _|   |___|   |___|  ...... |___|   |___|         *
* 													                              *
* 	LCD_CP Shift clock	1.388MHz (100MHz/36) 70Hz-es frissiteshez    *
\********************************************************************/
always @ ( posedge CLK_50MHz ) begin
	cp_clk_div <= cp_clk_div + 1;
	if ( cp_clk_div == 18 ) begin
		LCD_CP     <= LCD_CP + 1;
		cp_clk_div <= 0;
	end
end
//------------------------------------------------------------------


/********************************************************************\
* 	 LCD_LOAD	*								    	                        * 	
****************                                                     *
* 	                 >|       |< 740ns					                  *
* 		                ___     ___     _             ___	   __       *
*         	 CP     _|   |___|   |___|  ...... |___|   |___|         *
* 													                              *
* 		 		               ________	        	   ________	            *
* 		 LCD_LOAD     ______|        |___......__|        |____        *
* 													                              *
* 					         >||< 130ns					                     *
* 					                 >| |< 200ns		                     *
* 					          >|        |< 810ns				               *
\********************************************************************/
always @ ( posedge CLK_50MHz ) begin
	if ( cp_clk_div ==  6 && LCD_LOAD == 0 && lcd_x_ct == 79 && LCD_CP == 0 ) LCD_LOAD <= 1;
	if ( cp_clk_div == 10 && LCD_LOAD == 1 && lcd_x_ct == 79 && LCD_CP == 0 ) LCD_LOAD <= 0;
end
//------------------------------------------------------------------

/********************************************************************\
*	 LCD_FRAME	*    			                        					   *
****************	    						                              *
*	                  >|              |< 59.2us			               *
*		                 ______________		             ________      *
*      LCD_FRAME   ___|              |_______...... __|		         *
*													                              *
*		 		       	  ___	           ___ 	          ___	         *
*		 LCD_LOAD    ___|   |__________|   |___...... __|   |____      *
*													                              *
*				         >|   |< 810ns		    		                     *
*													                              *
\********************************************************************/
always @ ( posedge LCD_LOAD ) begin
	if ( lcd_y_ct == 0 && lcd_x_ct == 79 ) LCD_FRAME <= 1;
	if ( lcd_y_ct == 1 && lcd_x_ct == 79 ) LCD_FRAME <= 0;
	// display engedelyezes vagy tiltas 
	nLCD_DISP_OFF <= layer_enable[6] ;

end
//------------------------------------------------------------------


/********************************************************************\
*	LCD terites sor es oszlop szamlalok	 *  			    					*
*****************************************   									*
*													   									*
*	lcd_x_ct 0...79 ,  lcd_y_ct 0...239					   					*
\********************************************************************/
always @ ( posedge LCD_CP ) begin
	if ( lcd_x_ct == 79 ) begin
		lcd_x_ct <= 0;
		if ( lcd_y_ct == 239 ) lcd_y_ct <= 0; 	else	lcd_y_ct <= lcd_y_ct + 1; end
	else
			lcd_x_ct <= lcd_x_ct + 1;

	// 2 bit/pixel layer relativ RAM cimenek novelese
	if ( lcd_x_ct == 76 && lcd_y_ct == 239 ) grayscale_display_addr <= 0; else grayscale_display_addr <= grayscale_display_addr + 1;
	
	// 1 bit/pixel layer relativ RAM cimenek novelese
	if ( lcd_x_ct == 77 && lcd_y_ct == 239 ) display_addr <= 0; else	// layer relativ cim = 0 
		if ( lcd_x_ct[0] == 1 ) display_addr <= display_addr + 1;		// egyebkent noveli 1-el
end
//------------------------------------------------------------------

endmodule


// Looc-up tabla az arnyalatokhoz
//
module look_up_table( rom_addr, x, rom_data );
	input 	[3:0]		rom_addr;		// b3 b2 b1 b0
												//	 ---	^	 ^
												//	  |	|	 |
												//	  |	|	  -- lcd_x_ct
												//	  |	 ------ lcd_y_ct
												//	  |
												//		---------- layer_grayscale

	input		[1:0]		x;					// frame_ct

	output	[7:0]		rom_data;		// kimeneti ertek
	reg		[7:0]		rom_data;
	wire		[3:0]		nn;

	assign nn = { rom_addr[3:2], rom_addr[1:0] + x[1:0] };
	
	always @ ( nn ) begin
		case (nn)
			// 1 vilasos, 3 sotet
			 0: rom_data = 8'b00010001;
			 1: rom_data = 8'b10001000;
			 2: rom_data = 8'b01000100;
			 3: rom_data = 8'b00100010;
			// 2 vilagos, 2 sotet
			 4: rom_data = 8'b01010101;
			 5: rom_data = 8'b10101010;
			 6: rom_data = 8'b01010101;
			 7: rom_data = 8'b10101010;
			// 3 vilagos, 1 sotet
			 8: rom_data = 8'b01110111;
			 9: rom_data = 8'b11101110;
			10: rom_data = 8'b11011101;
			11: rom_data = 8'b10111011;
			// 4 vilagos, 0 sotet
			12: rom_data = 8'b11111111;
			13: rom_data = 8'b11111111;
			14: rom_data = 8'b11111111;
			15: rom_data = 8'b11111111;
		endcase
	end
endmodule


module rom3( rom_addr, x, rom_data );
	input 	[3:0]		rom_addr;		// b3 b2 b1 b0
												//	 ---	^	 ^
												//	  |	|	 |
												//	  |	|	  -- lcd_x_ct
												//	  |	 ------ lcd_y_ct
												//	  |
												//		---------- bit_grayscale

	input		[1:0]		x;					// frame_ct

	output				rom_data;		// kimeneti ertek
	reg					rom_data;
	wire		[3:0]		nn;

	assign nn = { rom_addr[3:2], rom_addr[1:0] + x[1:0] };
	
	always @ ( nn ) begin
		case (nn)
			// 1 vilasos, 3 sotet
			 0: rom_data = 0;
			 1: rom_data = 0;
			 2: rom_data = 0;
			 3: rom_data = 0;
			// 2 vilagos, 2 sotet
			 4: rom_data = 0;
			 5: rom_data = 1;
			 6: rom_data = 0;
			 7: rom_data = 1;
			// 3 vilagos, 1 sotet
			 8: rom_data = 0;
			 9: rom_data = 1;
			10: rom_data = 1;
			11: rom_data = 1;
			// 4 vilagos, 0 sotet
			12: rom_data = 1;
			13: rom_data = 1;
			14: rom_data = 1;
			15: rom_data = 1;
		endcase
	end
endmodule

module rom2( rom_addr, x, rom_data );
	input 	[3:0]		rom_addr;		// b3 b2 b1 b0
												//	 ---	^	 ^
												//	  |	|	 |
												//	  |	|	  -- lcd_x_ct
												//	  |	 ------ lcd_y_ct
												//	  |
												//		---------- bit_grayscale

	input		[1:0]		x;					// frame_ct

	output				rom_data;		// kimeneti ertek
	reg					rom_data;
	wire		[3:0]		nn;

	assign nn = { rom_addr[3:2], rom_addr[1:0] + x[1:0] };
	
	always @ ( nn ) begin
		case (nn)
			// 1 vilasos, 3 sotet
			 0: rom_data = 0;
			 1: rom_data = 0;
			 2: rom_data = 0;
			 3: rom_data = 0;
			// 2 vilagos, 2 sotet
			 4: rom_data = 1;
			 5: rom_data = 0;
			 6: rom_data = 1;
			 7: rom_data = 0;
			// 3 vilagos, 1 sotet
			 8: rom_data = 1;
			 9: rom_data = 1;
			10: rom_data = 1;
			11: rom_data = 0;
			// 4 vilagos, 0 sotet
			12: rom_data = 1;
			13: rom_data = 1;
			14: rom_data = 1;
			15: rom_data = 1;
		endcase
	end
endmodule

module rom1( rom_addr, x, rom_data );
	input 	[3:0]		rom_addr;		// b3 b2 b1 b0
												//	 ---	^	 ^
												//	  |	|	 |
												//	  |	|	  -- lcd_x_ct
												//	  |	 ------ lcd_y_ct
												//	  |
												//		---------- bit_grayscale

	input		[1:0]		x;					// frame_ct

	output				rom_data;		// kimeneti ertek
	reg					rom_data;
	wire		[3:0]		nn;

	assign nn = { rom_addr[3:2], rom_addr[1:0] + x[1:0] };
	
	always @ ( nn ) begin
		case (nn)
			// 1 vilasos, 3 sotet
			 0: rom_data = 0;
			 1: rom_data = 0;
			 2: rom_data = 0;
			 3: rom_data = 0;
			// 2 vilagos, 2 sotet
			 4: rom_data = 0;
			 5: rom_data = 1;
			 6: rom_data = 0;
			 7: rom_data = 1;
			// 3 vilagos, 1 sotet
			 8: rom_data = 1;
			 9: rom_data = 1;
			10: rom_data = 0;
			11: rom_data = 1;
			// 4 vilagos, 0 sotet
			12: rom_data = 1;
			13: rom_data = 1;
			14: rom_data = 1;
			15: rom_data = 1;
		endcase
	end
endmodule

module rom0( rom_addr, x, rom_data );
	input 	[3:0]		rom_addr;		// b3 b2 b1 b0
												//	 ---	^	 ^
												//	  |	|	 |
												//	  |	|	  -- lcd_x_ct
												//	  |	 ------ lcd_y_ct
												//	  |
												//		---------- bit_grayscale

	input		[1:0]		x;					// frame_ct

	output				rom_data;		// kimeneti ertek
	reg					rom_data;
	wire		[3:0]		nn;

	assign nn = { rom_addr[3:2], rom_addr[1:0] + x[1:0] };
	
	always @ ( nn ) begin
		case (nn)
			// 1 vilasos, 3 sotet
			 0: rom_data = 0;
			 1: rom_data = 0;
			 2: rom_data = 0;
			 3: rom_data = 0;
			// 2 vilagos, 2 sotet
			 4: rom_data = 1;
			 5: rom_data = 0;
			 6: rom_data = 1;
			 7: rom_data = 0;
			// 3 vilagos, 1 sotet
			 8: rom_data = 1;
			 9: rom_data = 0;
			10: rom_data = 1;
			11: rom_data = 1;
			// 4 vilagos, 0 sotet
			12: rom_data = 1;
			13: rom_data = 1;
			14: rom_data = 1;
			15: rom_data = 1;
		endcase
	end
endmodule


/********************************************************************\
*	Priorita encoder a RAM hozzaferesek engedelyezesehez	            *  			 
\********************************************************************/
 
module mem_prioritas_encoder(
	input_memreq , 	//  8-bit input 
	out 					//  5 bit output
  	);
  
  	output [4:0] out ;
  	reg 	 [4:0] out ;
  	input  [7:0] input_memreq ; 

	always @ ( input_memreq )
		begin 
			if ( input_memreq[0] == 1 ) 		out = 1;  // CPU olvasas
			else if ( input_memreq[1] == 1 ) out = 4;  // CPU iras
			else if ( input_memreq[2] == 1 ) out = 7;  // layer 1 olvasas
			else if ( input_memreq[3] == 1 ) out = 10; // layer 2 olvasas
			else if ( input_memreq[4] == 1 ) out = 13; // layer 3 olvasas
			else if ( input_memreq[5] == 1 ) out = 16; // layer 4 olvasas
			else if ( input_memreq[6] == 1 ) out = 19; // layer 5 olvasas
			else out = 0;
		end
endmodule 

