/************************************************************\
*                                                            *
*  					LCD kontroller V1.0                        *
*                                                            *
**************************************************************
*                                            					 *
* A  128 kbyte-os  RAM also es felso cimtartomanyanak kiosz- *
* tasa teljesen megegyezik, csak a lapozo  regiszterek erte- *
* ketol fuggoen van a terito logika szamara kijelolve.       *
* A lapozoregiszterektol fuggoen az adott layer vagy az elso *
* vagy a masodik 64 kbyte-os lapon helyezkedik el.           *
* A belso karaktergenerator tiltasa utan az  alapertelmezett *
* 1. karaktergenerator az elso  64 kbyte-os ram-ban van a 2. *
* pedig a masodik 64 kbyte-os lapon.                         *
* A teritett  karakter magassaga es  a  karaktergeneratorban *
* letarot karakterek hossza kulonbozhet. Nagyobb karakterme- *
* retnel a hianyzo karakterkep helyen extra ures sorok lesz- *
* nek megjelenitve.                                          *
*                                                            *
*      EGYSZERRE 512 FELE KARAKTERT LEHET MEGJELENITENI      *
*                                            					 *
**************************************************************
*                                                            *
*  DEC     0 -  9599  Layer 0 ( 1bit/pixel )						 *
*  DEC  9600 - 19199  Layer 1 ( 1bit/pixel )						 *
*  DEC 19200 - 28799  Layer 2 ( 1bit/pixel )						 * 
*  DEC 28800 - 38399  Layer 3 ( 1bit/pixel )						 *
*  DEC 38400 - 57599  Layer 4 ( 2bit/pixel )  					 *
*  DEC 57600 - 59999  Layer 5 ( karakteres )						 *
*							 A karakteres layer5 1200  byte hosszu, *
*                     de utana meg 1200 byte  van az  attri- *
*							 butumoknak  ezert  osszesen 2400 byte  *
*							 hosszu.											 *
*																				 *
*  DEC 60000 - 63840 EXT CGRAM 2./1. karaktergenerator		 *
*							A  karakter  merettol  fuggoen maximum  * 
*							3840  byte  hosszu  256*15 = 3840 byte	 *
*                    A belso  karaktergenerator vagy 8x8-as  *
*							vagy  8x12-es.  Ez attol fugg, hogy az	 *
*							FPGA flash beprogramozasakor  melyiket  *
*							rakom bele.									    *
* 							Ha a  belso  karaktergenerator  engede- *
*						   lyezve van, akkor a  2. kulso karakter- *
*                    generator  az 1.  lapon  a ram vegen a  *
*                    DEC 60000 cimnel kezdodik.              *
*                    Ha tiltva van, akkor az 1. lapon talal- *
*                    hato karaktergenerator lessz az alaper- *
*                    telmezett es a masodik lapon talalhato  *
*                    lessz a 2. karaktergenerator.           *
*                                                            *
* ************************************************************
*  LCD kijelzohoz      ***            RAM  csatlakozas       *
* ************************************************************
*  P3   LCD_DISP_OFF    |             P66  RAM_DATA0         *
*  P96  LCD_FRAME       |             P65  RAM_DATA1         *
*  P95  LCD_CP          |             P62  RAM_DATA2         *
*  P93  LCD_LOAD        |             P60  RAM_DATA3         *
*  P91  CLK_100MHz      |             P43  RAM_DATA4         *
*  P97  LCD_D3          |             P41  RAM_DATA5         * 
*  P87  LCD_D2          |             P54  RAM_DATA6         *
*  P86  LCD_D1          |             P59  RAM_DATA7   	    *
*  P84  LCD_D0          |                                    *
*                       |       		  P72  nRAM_OE           *
*                       |             P58  nRAM_WE           *
*______________________/              P68  nRAM_CE0          *
*  	                                                       *
*  P73  RAM_ADDRESS0                  P46  RAM_ADDRESS9      *
*  P71  RAM_ADDRESS1                  P44  RAM_ADDRESS10     *
*  P70  RAM_ADDRESS2                  P56  RAM_ADDRESS11     *
*  P69  RAM_ADDRESS3                  P52  RAM_ADDRESS12     *
*  P57  RAM_ADDRESS4                  P74  RAM_ADDRESS13     *
*  P55  RAM_ADDRESS5                  P80  RAM_ADDRESS14     *
*  P53  RAM_ADDRESS6                  P81  RAM_ADDRESS15     *
*  P47  RAM_ADDRESS7                  P82  RAM_ADDRESS16     *
*  P45  RAM_ADDRESS8                                         *
* ___________________________________________________________*
*                   *                                        *
*  CPU csatlakozas  *                                        *
* *******************                                        *
*                                                            *
*  P4   nCPU_RD                       P6   CPU_ADDRESS0		 *
*	P5   nCPU_WR  	                    P40  CPU_ADDRESS1      *
*  P6                                 P7   CPU_ADDRESS2      *
*  P7                                                        *
*  P8                                                        *
*                                                            *
*  P20  CPU_DATA0                                            *
*  P21  CPU_DATA1                                            *
*  P22  CPU_DATA2                                            *
*  P30  CPU_DATA3                                            *
*  P31  CPU_DATA4                                            *
*  P32  CPU_DATA5                                            *
*  P34  CPU_DATA6                                            *
*  P83  CPU_DATA7                                            *
*                                                            *
\************************************************************/

module LCD(
	CLK_100MHz,
	LCD_DATA,
	LCD_FRAME,
	LCD_LOAD,
	LCD_CP,
	nLCD_DISP_OFF,
	RAM_ADDRESS,
	RAM_DATA,
	nRAM_CE0,
	nRAM_OE,
	nRAM_WE,	
	nCPU_RD,
	nCPU_WR,
	CPU_DATA,
	CPU_ADDRESS,
	PROBA
);

output PROBA;
reg PROBA = 0;

// *** RAM kivezetesekhez tartozo delkaraciok ***
//
output [16:0]	RAM_ADDRESS;		// RAM cim
reg	 [16:0]	RAM_ADDRESS = 0;
inout	 [7:0] 	RAM_DATA;			// RAM adat
output 			nRAM_CE0;			// RAM CE0
reg				nRAM_CE0 	= 1;
output			nRAM_OE;	    		// RAM OE
reg				nRAM_OE  	= 1;
output			nRAM_WE;				// RAM WE
reg 				nRAM_WE  	= 1;


// *** LCD kivezetesekhez tartozo deklaraciok ***
//
output [3:0]   LCD_DATA; 	   	   // LCD 4 bit adat
reg    [3:0]   LCD_DATA 		= 0;
output			LCD_FRAME;	      	// LCD V sync.
reg				LCD_FRAME 		= 0;
output			LCD_LOAD;		   	// LCD H sync
reg				LCD_LOAD 		= 0;
output			LCD_CP;		      	// LCD clock
reg				LCD_CP 			= 0;
output			nLCD_DISP_OFF;			// LCD engedelyezes=1, tiltas=0
reg				nLCD_DISP_OFF	=0;

// *** CPU kivezetesekhez tartozo deklaraciok ***
//
input				nCPU_RD;	 			// CPU RD jel aktiv "L" szint
input				nCPU_WR;				// CPU WR jel aktiv "L" szint
input	[2:0]		CPU_ADDRESS;		// CPU felol A0, A1, A2 cimvezetek
inout	[7:0]		CPU_DATA;			// CPU felol 8bit-es adatbusz


// *** Orajelbemenethez es eloosztokhoz tartozo deklaraciok ***
//
input			   CLK_100MHz;	      // 100MHz orajel
reg   [6:0]		cp_clk_div = 0;	// Elooszto a shift clock jelhez

// *** Eldetektalashoz es komparatorokhoz tartozo deklaraciok ***
//
reg				lcd_cp_0_felfutas 				= 0;	// Az LCD CP 1. periodusanak felfuto ele eseten
																	// 1 orajelig 1-be irodik
reg				lcd_cp_1_felfutas 				= 0;	// Az LCD CP 2. periodusanak felfuto ele eseten
																	// 1 orajelig 1-be irodik
reg				lcd_cp_ct							= 0;	// Az LCD CP jel billegteti, az elozo ket impulsud
																	// eloallitasara hasznalom
reg	[1:0]		ncpu_rd_edge 	   				= 0;	// CPU RD jel eldetektalasahoz mintaveteli shift regiszter
reg	[1:0]		ncpu_wr_edge 	   				= 0;	// CPU WR jel eldetektalasahoz mintaveteli shift regiszter
reg				lcd_y_ct_max	   				= 0; 	// Ha az lcd_y_ct = 239 akkor 1 egyebkent 0.
reg				lcd_x_ct_max						= 0;	// Ha az lcd_x_ct =  79 akkor 1 egyebkent 0.
reg				car_display_addr_1199			= 0;	// Ha a car_display_addr = 1199 akkor 1 egyebkent 0

// *** State machine deklaraciok ***
//

reg	[1:0]		cpu_cgram_cikl		= 0; 	// CPU CGRAM olvasasi ciklus

reg	[4:0]		ram_cikl			   = 0;	// 0 nem hasznalja senki a RAM-ot
													// 1..3 CPU ir a video RAM-ba
													// 4..6 ????
													// 7..9   layer 1 olvasasa a video RAM-bol
													// 10..12 layer 2 olvasasa a video RAM-bol
													// 13..15 layer 3 olvasasa a video RAM-bol
													// 16..18 layer 4 olvasasa a video RAM-bol
													// 19..21 layer 5 olvasasa a video RAM-bol
													// 22..24 layer 6 olvasasa a video RAM-bol

// *** Egyeb belso regiszterek ***
//
reg 				rambde			 = 1;		// A RAM irasakor a ezzel van engedelyeze a kimeneti 
													// buszmeghajto a kivezetesek egyseges idoyitese miatt.
reg	[7:0]		cpu_cgram_data	 = 0;		// A CPU CGRAM-bol olvasott adat
reg	[7:0]		ram_write_data  = 0;		// RAM-ba irando adat
reg	[7:0]		cpu_read_data   = 0;		// CPU_ADDRESS = 0 - RAM data --> CPU data
reg	[7:0]		cpu_write_data  = 0;		// CPU_ADDRESS = 0 - CPU data <-- RAM data
reg	[15:0]	cpu_ram_address = 0;		// A CPU RAM cimregisztere
reg				display_inv		 = 0; 	// Dsiplay invertalas 0=normal, 1=inverz

// A CPU RAM cimzesekor es az egyes layer-ek A16-os cimet allitja be.
reg				cpu_ram_A16	= 0;
reg				layer1_A16	= 0;	
reg				layer2_A16	= 0;	
reg				layer3_A16	= 0;	
reg				layer4_A16	= 0;	
reg				layer5_A16	= 0;	
reg				layer6_A16	= 0;	
 
reg	[7:0]		layer_grayscale = 0;		// Az 1bit/pixel-es layer-ek fenyerejenek erteket tartalmazza
													// b7 b6 b5 b4 b3 b2 b1 b0 <--- layer_grayscale regiszter
													//  ---	 ---	 ---	 ---
													//	  |     |     |	  |
													//	  |     |	  |		------->	Layer 1 
													//   |	  |		------------->	Layer 2 
													//   |	   ------------------->	Layer 3
													//    ------------------------->	Layer 4
													//
													// 2 bit/layer 
													//		0 = 1 vilagos 3 sotet
													//		1 = 2 vilagos 2 sotet
													//		2 = 3 vilagos 1 sotet
													//		3 = 4 vilagos 0 sotet
													
reg	[7:0]		lcd_layer_data_puffer = 0;	// Ebbe mixeli ossze a kulonbozo layer-ek adatait
reg	[3:0]		lcd_out_puff			 = 0;	// Az elozo LCD CP ciklusban olvasott byte felso 4 bit-jet 
														// tarolom benne
														
reg	[1:0]		yx							 = 0; // A look-up tabla cimkeveresere kell
														// ( eredetileg az lcd_y_ct[0] es az lcd_x_ct[0] volt, de
														// a 2 LCD CP ideig tarto ram ciklusok miatt fuggetleniteni kellett )

// *** Ram hozzaferest kero jelzobitek ***
//

reg	cpu_cgram_mem_req	= 0;			// CGRAM olvasas a CPU fele

reg	[7:0] mem_req		= 0;			// 	b7 b6 b5 b4 b3 b2 b1 b0 --->  mem_req regiszter
												//	  	 |  |  |  |  |  |  |  |
												//	  	 |  |  |	 |  |  |  |   ---->	CPU RAM olvasas
												//     |	 |  |  |  |  |   ------->	CPU RAM iras
												//     |	 |  |  |	 |   ---------->	Layer 1 ( 1bit/pixel	) olvasas
												//     |  |  |  |   ------------->  Layer 2 ( 1bit/pixel	) olvasas
												//     |  |  |   ---------------->	Layer 3 ( 1bit/pixel	) olvasas
												//		 |	 |   ------------------->  Layer 4 ( 1bit/pixel	) olvasas
												//		 |	  ---------------------->  Layer 5 ( 2bit/pixel	) olvasas
												//		  ------------------------->	Layer 6 ( karakteres	) olvasas


// *** Layer engedelyezo bitek ***
//			
// A layer_enable regisztert a CPU barmikor 
// valtoztathatja, de a szamozott engedelyezo
// bitek csak minden kepvaltaskor modosulnak.
//
reg	[7:0]	layer_enable 	 	= 0;	   // Layer engedelyezo regiszter
reg			layer1_enable	 	= 0;		// Layer 1 engedelyezo regiszter 0=disable, 1=enable
reg			layer2_enable	 	= 0;		// Layer 2 engedelyezo regiszter 0=disable, 1=enable
reg			layer3_enable	 	= 0;		// Layer 3 engedelyezo regiszter 0=disable, 1=enable
reg			layer4_enable	 	= 0;		// Layer 4 engedelyezo regiszter 0=disable, 1=enable
reg			layer5_enable	 	= 0;		// Layer 5 engedelyezo regiszter 0=disable, 1=enable
reg			layer6_enable	 	= 0;		// Layer 6 engedelyezo regiszter 0=disable, 1=enable
reg			auto_increment_en = 0;		// Automatikus cimnoveles engedelyezes

// *** Szamlalokhoz tartozo deklaraciok ***
//
reg	[13:0]	display_addr	 		  	= 0;	// A display terites soran az 1 bit/pixel-es kepterites relativ RAM cimet 
															// tartalmazza ami minden masodik LCD shift clock utan novel. ( 0...9599 )
															// Azert minden masodikra mert az video RAM 8 bit-es, de az LCD adatbusz 
															// csak 4bit-es es igy egy RAM olvasara 2 LCD buszciklus jut.
															
reg 	[14:0]	grayscale_display_addr	= 0;	// A display terites soran a 2 bit/pixel-es kepterites relativ RAM cimet 
															// tartalmazza ami minden masodik LCD shift clock utan novel. ( 0...9599 )
reg				grayscale_ovf 				= 0;	// Ha a grayscale_display_addr elerte a maximalis erteket , akkor
															// 1-be irodik. Ez jelzi a display terito resznek, hogy a masodik
															// byte felolvasasat a 0. cimrol kell elvegezni.
															// Ez azert van, mert a 2bit/pixel-es layer 1 LCD CP ciklussal
															// el van csuszva, de nem tudom miert. Emiatt van ez a kavaras. :-(

reg   [6:0]		lcd_x_ct 					= 0;	// LCD oszlop szamlalo 0..78  ( 80 x 4 bit 320 pixel )
reg	[7:0]		lcd_y_ct 					= 0;	// LCD sor szamlalo    0..239 ( 240 sor )

reg				cpu_byte_ct					= 0;	// A ket byte-os CPU RAM irasnal a felolvasott byte-ot szamolja 			
reg				grayscale_byte_ct 		= 0;	// A ket byte-os Layer5 RAM olvasasnal a felolvasott byte-ot szamolja 			
reg	[5:0]		frame_ct 				 	= 0;	// minden 240. sor teritese utan novekszik
															// "ROM" tablazat cimenek offsetelesere hasznalom az also ket bitet
															// igy megkavarja a kijelzest 4 egymast kiveto sorban es 4 egymas utan
															// kovetkezo kep utan. 
															// A villogtatashoz a legfelso bit van hasznalva
															
reg	[10:0]	car_display_addr			= 0;	// A display terites soran a karakteres layer relativ
                                             // ram cimet tartalmazza ( 0...1199 )
reg	[1:0]		car_byte_ct					= 0;  // A karakteres display RAM olvasaskor a felolvasott byte-okat
															// szamolom benne. ( 2 vagy 3 byte kell )
reg	[3:0]		car_line_ct					= 1;  // A karakterterites soran a karakteren beluli 
															// sorokat szamolja 1...15
reg	[5:0]		car_x_ct						= 0;	// Karakter terites oszlop szamlalo 0...39
reg	[2:0]		car_size						= 4;	// Karakter magassag 8...15 pixel
															// 0 eseten 8 pixel ... 7 eseten 15 pixel
reg				ext_car_gen					= 0;	// Az 1. karaktergenerator helye 0=int BlockRAM, 1=ext ViedoRAM
reg	[2:0]		car_gen_size				= 4;	// A karaktergenerator karaktermagassaga 8...15 pixel
															// 0 eseten 8 pixel ... 7 eseten 15 pixel
reg	[7:0]		car_data						= 0;	// A karakter kodjat tartalmazza amig a karaktergeneratort
															// cimzi a terito logika. Utana pedig a karaktergeneratorbol
															// kiolvasott bitmintat.
reg	[7:0]		car_attr						= 0;	// A karakter attributum a karakter terites alatt
reg				extra_line					= 0;	// Ha a karakterterites soran a karaktergeneratorban
															// tarolt karakter kissebb mint a terites soran a
															// car_line_ct, akkor 1-be irom. Ezzel jelzem a logikanak,
															// hogy nincs tobb ervenyes karaktergenerator bitminta.
reg	[1:0]		car_read_ok_ct				= 0;	// A karakteres layer teritesekor ha a car_data mar
															// tartalmazza a karakternek megfelelo bitmintat, akkor
															// az erteke 1 lessz. Ezt hasznalom fel egy masik "case"
															// feltetel vizsgalatkor hogy az lcd-re kerulo adatot
															// aloallitsam. Azert kell hogy ne fogja meg addig a
															// RAM kezelo logikat. Az lcd adatpufferbe 3 orajel mulva
															// irja be a megfelelo bitmintat.
															
reg	[7:0]		cpu_car_attr				= 0;	// Ha a write_car_attr=1 akkor  a karakteres
															// ram irasakor az irott karakterhez tartozo
															// attributum cimere irja ennek a regiszternek a tartalmat.
reg				write_car_attr				= 0;	// Automatikus karakter attributum iras engedelyezes
reg				attr_write_en				= 0;	// Ha a cpu_ram_address nagyobb mint a legutolso 
															// karakteres layer cime, akkor engedi az 1200
															// byte-al feljebb levo attributum byte automatikus
															// irasat.
// *** Constans deklaraciok ***
//

parameter Felfuto_el = 1; // a 2bit-es shift regiszterek tartalma 01 ha felfuto el volt
parameter Lefuto_el  = 2; // a 2bit-es shift regiszterek tartalma 10 ha felfuto el volt


// Layer alap kezdocimek
//
parameter	[15:0]	layer1_start	= 0;		//  9600 byte hosszu
parameter	[15:0]	layer2_start	= 9600;	//  9600 byte hosszu
parameter	[15:0]	layer3_start	= 19200;	//  9600 byte hosszu
parameter	[15:0]	layer4_start	= 28800;	//  9600 byte hosszu
parameter	[15:0]	layer5_start	= 38400;	// 19200 byte hosszu
parameter	[15:0]	layer6_start	= 57600;	//  2400 byte hosszu mert az attributum miatt 2 byte kell
parameter	[15:0]	cg_ram_start	= 60000; // A karakter merettol fuggoen maximum 3840 byte hosszu 
															// 256*15 = 3840 byte EXT CGRAM
															            


// *** Modulokhoz szukseges deklaraciok ***
//

// Az 1bit/pixel-es ROM-hoz valo deklaraciok
reg	[3:0]		look_up_table_addr 	= 0;		// A fenyero "ROM" tablazat cime
wire	[7:0]		look_up_table_data;				// A fenyero "ROM" tablazat kimeno adat

// A 2bit/pixel-es ROM-hoz valo deklaraciok
reg	[3:0]		rom0_addr 				= 0;		// A  0. fenyero "ROM" tablazat cime
wire				rom0_data;							// A  0. fenyero "ROM" tablazat kimeno adat
reg	[3:0]		rom1_addr 				= 0;		// Az 1. fenyero "ROM" tablazat cime
wire				rom1_data;							// Az 1. fenyero "ROM" tablazat kimeno adat
reg	[3:0]		rom2_addr 				= 0;		// A  2. fenyero "ROM" tablazat cime
wire				rom2_data;							// A  2. fenyero "ROM" tablazat kimeno adat
reg	[3:0]		rom3_addr 				= 0;		// A  3. fenyero "ROM" tablazat cime
wire				rom3_data;							// A  3. fenyero "ROM" tablazat kimeno adat


wire	[4:0]	mem_req_start; 	// Memoria hozzaferes prioritasdekoderenek a kimenete
wire	[1:0] layer_num;			// A prioritas dekoderben kap erteket az 1bit/pixel-es
										// layer sorszamanak megfeleloen 0...3 kozotti ertekben.

wire	ickosc, CLK100, ck100, ck50, CLK_50MHz; // A belso DLL-ez szukseges deklaraciok

	look_up_table TABLA1( .rom_addr( look_up_table_addr ), .x( frame_ct[1:0] ), .rom_data( look_up_table_data ) );
	rom0 TABLA2( .rom_addr( rom0_addr ), .x( frame_ct[1:0] ), .rom_data( rom0_data ) );
	rom1 TABLA3( .rom_addr( rom1_addr ), .x( frame_ct[1:0] ), .rom_data( rom1_data ) );
	rom2 TABLA4( .rom_addr( rom2_addr ), .x( frame_ct[1:0] ), .rom_data( rom2_data ) );
	rom3 TABLA5( .rom_addr( rom3_addr ), .x( frame_ct[1:0] ), .rom_data( rom3_data ) );
	mem_prioritas_encoder PRIENC1( .input_memreq( mem_req ), .out( mem_req_start ), .out_layer_num( layer_num ) );

reg	[11:0]	cgram_addr = 0;	// A karakteres layer teritesekor a ram relativ cime van benne
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// A belso blokk RAM -hoz valo deklaraciok 
//
wire	[7:0]	CGRAM0_DATA;  			// Port A 8-bit data output
wire	[7:0]	CGRAM1_DATA;  			// Port A 8-bit data output
wire	[7:0]	CGRAM2_DATA;  			// Port A 8-bit data output
wire	[7:0]	CGRAM3_DATA;  			// Port A 8-bit data output
wire	[7:0]	CGRAM4_DATA;  			// Port A 8-bit data output
wire	[7:0]	CGRAM5_DATA;  			// Port A 8-bit data output
wire	[7:0]	CPU_CGRAM_DATA0;		// Port B 8-bit data output
wire	[7:0]	CPU_CGRAM_DATA1;		// Port B 8-bit data output
wire	[7:0]	CPU_CGRAM_DATA2;		// Port B 8-bit data output
wire	[7:0]	CPU_CGRAM_DATA3;		// Port B 8-bit data output
wire	[7:0]	CPU_CGRAM_DATA4;		// Port B 8-bit data output
wire	[7:0]	CPU_CGRAM_DATA5;		// Port B 8-bit data output
reg	[8:0] CGRAM_ADDR 		= 0;	// Port A 9-bit address input
reg	[8:0] CPU_CGRAM_ADDR	= 0;	// Port B 9-bit address input
reg			CGRAM_CLK   	= 0;	// Port A clock input
reg		   CPU_CGRAM_CLK	= 0;	// Port B clock input
reg	[7:0] DIA     			= 0;	// Port A 8-bit data input
reg	[7:0] DIB     			= 0;	// Port B 8-bit data input
reg		   CGRAM_EN    	= 0;	// Port A RAM enable input
reg		   CPU_CGRAM_EN	= 0;	// Port B RAM enable input
reg		   RSTA   			= 0;	// Port A Synchronous reset input
reg		   RSTB   			= 0;	// Port B Synchronous reset input
reg		   CGRAM_WE    	= 0;	// Port A RAM write enable input
reg		   CPU_CGRAM_WE	= 0;	// Port B RAM write enable input

// Blokk ram-bam a karaktergenerator
//
// RAMB4_S8_S8: Virtex/E, Spartan-II/IIE 512 x 8 Dual-Port RAM
// The current version of the Xilinx HDL Libraries Guide
//
   RAMB4_S8_S8 RAM0_inst (
      .DOA(CGRAM0_DATA),   // Port A 8-bit data output
      .DOB(CPU_CGRAM_DATA0),     		// Port B 8-bit data output
      .ADDRA(CGRAM_ADDR), 	// Port A 9-bit address input
      .ADDRB(CPU_CGRAM_ADDR), 		// Port B 9-bit address input
      .CLKA(CGRAM_CLK),   	// Port A clock input
      .CLKB(CPU_CGRAM_CLK),   		// Port B clock input
      .DIA(DIA),     		// Port A 8-bit data input
      .DIB(DIB),     		// Port B 8-bit data input
      .ENA(CGRAM_EN),      // Port A RAM enable input
      .ENB(CPU_CGRAM_EN),     		// Port B RAM enable input
      .RSTA(RSTA),   		// Port A Synchronous reset input
      .RSTB(RSTB),   		// Port B Synchronous reset input
      .WEA(CGRAM_WE),     	// Port A RAM write enable input
      .WEB(CPU_CGRAM_WE)      		// Port B RAM write enable input
   );

   RAMB4_S8_S8 RAM1_inst (
      .DOA(CGRAM1_DATA),   // Port A 8-bit data output
      .DOB(CPU_CGRAM_DATA1),     		// Port B 8-bit data output
      .ADDRA(CGRAM_ADDR), 	// Port A 9-bit address input
      .ADDRB(CPU_CGRAM_ADDR), 		// Port B 9-bit address input
      .CLKA(CGRAM_CLK),   	// Port A clock input
      .CLKB(CPU_CGRAM_CLK),   		// Port B clock input
      .DIA(DIA),     		// Port A 8-bit data input
      .DIB(DIB),     		// Port B 8-bit data input
      .ENA(CGRAM_EN),      // Port A RAM enable input
      .ENB(CPU_CGRAM_EN),     		// Port B RAM enable input
      .RSTA(RSTA),   		// Port A Synchronous reset input
      .RSTB(RSTB),   		// Port B Synchronous reset input
      .WEA(CGRAM_WE),     	// Port A RAM write enable input
      .WEB(CPU_CGRAM_WE)      		// Port B RAM write enable input
   );

   RAMB4_S8_S8 RAM2_inst (
      .DOA(CGRAM2_DATA),   // Port A 8-bit data output
      .DOB(CPU_CGRAM_DATA2),     		// Port B 8-bit data output
      .ADDRA(CGRAM_ADDR), 	// Port A 9-bit address input
      .ADDRB(CPU_CGRAM_ADDR), 		// Port B 9-bit address input
      .CLKA(CGRAM_CLK),   	// Port A clock input
      .CLKB(CPU_CGRAM_CLK),   		// Port B clock input
      .DIA(DIA),     		// Port A 8-bit data input
      .DIB(DIB),     		// Port B 8-bit data input
      .ENA(CGRAM_EN),      // Port A RAM enable input
      .ENB(CPU_CGRAM_EN),     		// Port B RAM enable input
      .RSTA(RSTA),   		// Port A Synchronous reset input
      .RSTB(RSTB),   		// Port B Synchronous reset input
      .WEA(CGRAM_WE),     	// Port A RAM write enable input
      .WEB(CPU_CGRAM_WE)      		// Port B RAM write enable input
   );

   RAMB4_S8_S8 RAM3_inst (
      .DOA(CGRAM3_DATA),   // Port A 8-bit data output
      .DOB(CPU_CGRAM_DATA3),     		// Port B 8-bit data output
      .ADDRA(CGRAM_ADDR), 	// Port A 9-bit address input
      .ADDRB(CPU_CGRAM_ADDR), 		// Port B 9-bit address input
      .CLKA(CGRAM_CLK),   	// Port A clock input
      .CLKB(CPU_CGRAM_CLK),   		// Port B clock input
      .DIA(DIA),     		// Port A 8-bit data input
      .DIB(DIB),     		// Port B 8-bit data input
      .ENA(CGRAM_EN),      // Port A RAM enable input
      .ENB(CPU_CGRAM_EN),     		// Port B RAM enable input
      .RSTA(RSTA),   		// Port A Synchronous reset input
      .RSTB(RSTB),   		// Port B Synchronous reset input
      .WEA(CGRAM_WE),     	// Port A RAM write enable input
      .WEB(CPU_CGRAM_WE)      		// Port B RAM write enable input
   );

   RAMB4_S8_S8 RAM4_inst (
      .DOA(CGRAM4_DATA),   // Port A 8-bit data output
      .DOB(CPU_CGRAM_DATA4),     		// Port B 8-bit data output
      .ADDRA(CGRAM_ADDR), 	// Port A 9-bit address input
      .ADDRB(CPU_CGRAM_ADDR), 		// Port B 9-bit address input
      .CLKA(CGRAM_CLK),   	// Port A clock input
      .CLKB(CPU_CGRAM_CLK),   		// Port B clock input
      .DIA(DIA),     		// Port A 8-bit data input
      .DIB(DIB),     		// Port B 8-bit data input
      .ENA(CGRAM_EN),      // Port A RAM enable input
      .ENB(CPU_CGRAM_EN),     		// Port B RAM enable input
      .RSTA(RSTA),   		// Port A Synchronous reset input
      .RSTB(RSTB),   		// Port B Synchronous reset input
      .WEA(CGRAM_WE),     	// Port A RAM write enable input
      .WEB(CPU_CGRAM_WE)      		// Port B RAM write enable input
   );

   RAMB4_S8_S8 RAM5_inst (
      .DOA(CGRAM5_DATA),   // Port A 8-bit data output
      .DOB(CPU_CGRAM_DATA5),     		// Port B 8-bit data output
      .ADDRA(CGRAM_ADDR), 	// Port A 9-bit address input
      .ADDRB(CPU_CGRAM_ADDR), 		// Port B 9-bit address input
      .CLKA(CGRAM_CLK),   	// Port A clock input
      .CLKB(CPU_CGRAM_CLK),   		// Port B clock input
      .DIA(DIA),     		// Port A 8-bit data input
      .DIB(DIB),     		// Port B 8-bit data input
      .ENA(CGRAM_EN),      // Port A RAM enable input
      .ENB(CPU_CGRAM_EN),     		// Port B RAM enable input
      .RSTA(RSTA),   		// Port A Synchronous reset input
      .RSTB(RSTB),   		// Port B Synchronous reset input
      .WEA(CGRAM_WE),     	// Port A RAM write enable input
      .WEB(CPU_CGRAM_WE)      		// Port B RAM write enable input
   );

//*****************************************************************
//    8 x 12 font keszlet
//*****************************************************************
defparam RAM0_inst.INIT_00 = 256'hE7C3FFFFDBFF7E0000007E8199BD8181A5817E00000000000000000000000000;
defparam RAM0_inst.INIT_01 = 256'h3C3C180000000010387CFE7C38100000000010387CFEFEFEFE6C000000007EFF;
defparam RAM0_inst.INIT_02 = 256'h00000000183C3C180000000000003C18187EFFFF7E3C180000003C1818E7E7E7;
defparam RAM0_inst.INIT_03 = 256'h99BDBD99C3FFFFFF0000003C664242663C000000FFFFFFFFE7C3C3E7FFFFFFFF;
defparam RAM0_inst.INIT_04 = 256'h3F333F00000018187E183C6666663C00000078CCCCCC78321A0E1E00FFFFFFC3;
defparam RAM0_inst.INIT_05 = 256'h00001818DB3CE73CDB18180000C0E6E7676363637F637F000000E0F070303030;
defparam RAM0_inst.INIT_06 = 256'h7E1818187E3C1800000002060E3EFE3E0E060200000080C0E0F8FEF8E0C08000;
defparam RAM0_inst.INIT_07 = 256'h3860C67C00001B1B1B1B7BDBDBDB7F000000666600666666666666000000183C;
defparam RAM0_inst.INIT_08 = 256'h007E183C7E1818187E3C18000000FEFEFE000000000000007CC60C386CC6C66C;
defparam RAM0_inst.INIT_09 = 256'h180CFE0C180000000000183C7E1818181818180000001818181818187E3C1800;
defparam RAM0_inst.INIT_0A = 256'h2800000000000000FEC0C0C000000000000000003060FE603000000000000000;
defparam RAM0_inst.INIT_0B = 256'h0000001038387C7CFEFE0000000000FEFE7C7C383810000000000000286CFE6C;
defparam RAM0_inst.INIT_0C = 256'h0000000024666666000018180018183C3C3C1800000000000000000000000000;
defparam RAM0_inst.INIT_0D = 256'hC200000018187CC686067CC0C2C67C1800006C6CFE6C6C6CFE6C6C0000000000;
defparam RAM0_inst.INIT_0E = 256'h000000000000000060303030000076CCCCDC76386C6C38000000C66630180CC6;
defparam RAM0_inst.INIT_0F = 256'h663CFF3C66000000000030180C0C0C0C0C18300000000C183030303030180C00;

defparam RAM1_inst.INIT_00 = 256'h000000000030181818000000000000000000000018187E181800000000000000;
defparam RAM1_inst.INIT_01 = 256'h00000080C06030180C060200000018180000000000000000000000000000FE00;
defparam RAM1_inst.INIT_02 = 256'h6030180C06C67C0000007E18181818187838180000007CC6C6E6F6DECEC67C00;
defparam RAM1_inst.INIT_03 = 256'hC0C0FE0000001E0C0CFECC6C3C1C0C0000007CC606063C0606C67C000000FEC6;
defparam RAM1_inst.INIT_04 = 256'h000030303030180C06C6FE0000007CC6C6C6FCC0C060380000007CC60606FCC0;
defparam RAM1_inst.INIT_05 = 256'h18000000181800000000780C06067EC6C6C67C0000007CC6C6C67CC6C6C67C00;
defparam RAM1_inst.INIT_06 = 256'h000000000000060C18306030180C060000003018180000001818000000000018;
defparam RAM1_inst.INIT_07 = 256'h000018180018180CC6C67C0000006030180C060C18306000000000007E00007E;
defparam RAM1_inst.INIT_08 = 256'h66667C666666FC000000C6C6C6FEC6C66C38100000007CC0DCDEDEDEC6C67C00;
defparam RAM1_inst.INIT_09 = 256'h6266FE000000F86C66666666666CF80000003C66C2C0C0C0C2663C000000FC66;
defparam RAM1_inst.INIT_0A = 256'h00003A66C6DEC0C0C2663C000000F060606878686266FE000000FE6662687868;
defparam RAM1_inst.INIT_0B = 256'hCC0C0C0C0C0C1E0000003C181818181818183C000000C6C6C6C6FEC6C6C6C600;
defparam RAM1_inst.INIT_0C = 256'hFEEEC6000000FE66626060606060F0000000E6666C6C786C6C66E600000078CC;
defparam RAM1_inst.INIT_0D = 256'h0000386CC6C6C6C6C66C38000000C6C6C6CEDEFEF6E6C6000000C6C6C6C6D6FE;
defparam RAM1_inst.INIT_0E = 256'h666C7C666666FC00000E0C7CDED6C6C6C6C67C000000F06060607C666666FC00;
defparam RAM1_inst.INIT_0F = 256'hC6C6C60000003C18181818185A7E7E0000007CC6C60C3860C6C67C000000E666;

defparam RAM2_inst.INIT_00 = 256'h00006C7CFED6D6C6C6C6C600000010386CC6C6C6C6C6C60000007CC6C6C6C6C6;
defparam RAM2_inst.INIT_01 = 256'hC26030188CC6FE0000003C1818183C66666666000000C6C66C3838386CC6C600;
defparam RAM2_inst.INIT_02 = 256'h0C0C3C00000002060E1C3870E0C0800000003C303030303030303C000000FEC6;
defparam RAM2_inst.INIT_03 = 256'hFF0000000000000000000000000000000000000000C66C3800003C0C0C0C0C0C;
defparam RAM2_inst.INIT_04 = 256'h66666C786060E000000076CCCC7C0C7800000000000000000000000000001830;
defparam RAM2_inst.INIT_05 = 256'h00000000000076CCCCCC6C3C0C0C1C0000007CC6C0C0C67C0000000000007C66;
defparam RAM2_inst.INIT_06 = 256'h78CC0C7CCCCCCC76000000000000F0606060F060646C380000007CC6C0FEC67C;
defparam RAM2_inst.INIT_07 = 256'h0606060E0006060000003C1818181838001818000000E6666666766C6060E000;
defparam RAM2_inst.INIT_08 = 256'h0000000000003C1818181818181838000000E6666C786C666060E0003C666606;
defparam RAM2_inst.INIT_09 = 256'h00007CC6C6C6C67C0000000000006666666666DC000000000000C6D6D6D6FEEC;
defparam RAM2_inst.INIT_0A = 256'h606676DC000000001E0C0C7CCCCCCC7600000000F060607C666666DC00000000;
defparam RAM2_inst.INIT_0B = 256'h0000000000001C36303030FC3030100000007CC61C70C67C000000000000F060;
defparam RAM2_inst.INIT_0C = 256'h00006CFED6D6C6C6000000000000183C6666666600000000000076CCCCCCCCCC;
defparam RAM2_inst.INIT_0D = 256'h3018CCFE00000000F80C067EC6C6C6C6000000000000C66C38386CC600000000;
defparam RAM2_inst.INIT_0E = 256'h1818700000001818181800181818180000000E181818701818180E000000FE66;
defparam RAM2_inst.INIT_0F = 256'h000000FEC6C66C3810000000000000000000000000DC76000000701818180E18;

defparam RAM3_inst.INIT_00 = 256'h6E2010E8246220E00E080E824E2010E8444040C02E2A2EA86E2010E8444240C0;
defparam RAM3_inst.INIT_01 = 256'h246220E02E2A2EA86E2010E824E280E002020E8A4A2010E8444240C02E2A2EA8;
defparam RAM3_inst.INIT_02 = 256'h0E080E824E2010E8444240C02E2A2EA86E201048444220E00E0A0E8A4E2010E8;
defparam RAM3_inst.INIT_03 = 256'h6E2010A8A4A2A0A00E0A0E8A4E2010E824E280E02E2A2EA86E20102824E2A0E0;
defparam RAM3_inst.INIT_04 = 256'h24E280E02E2A2EA86E2010B894B290B002020E8A4A2010E8246220E02E2A2EA8;
defparam RAM3_inst.INIT_05 = 256'h0000C6C6FEC6C66C3800386C2E2A2EA86E2010B88CBAA0B80E0A0E8A4E2010E8;
defparam RAM3_inst.INIT_06 = 256'hCCCCFECCCC6C3E0000006ED8D87E3676CC0000000000FE66607C6066FE006030;
defparam RAM3_inst.INIT_07 = 256'h0018306000007CC6C6C6C67C00C6C60000007CC6C6C6C67C006C38100000CECC;
defparam RAM3_inst.INIT_08 = 256'h000076CCCCCCCCCC00183060000076CCCCCCCCCC00CC783000007CC6C6C6C67C;
defparam RAM3_inst.INIT_09 = 256'hC6C6C6C6C600C6C60000386CC6C6C6C66C38C6C6780C067EC6C6C6C600C6C600;
defparam RAM3_inst.INIT_0A = 256'h3C6666000000FCE6606060F060646C38000018183C666060663C181800007CC6;
defparam RAM3_inst.INIT_0B = 256'h70D8181818187E1818181B0E0000C6CCCCDECCC4F8CCCCF8000018187E187E18;
defparam RAM3_inst.INIT_0C = 256'hC6C6C67C0060301800003C18181818380030180C000076CCCC7C0C7800603018;
defparam RAM3_inst.INIT_0D = 256'hE6C600DC00006666666666DC00DC7600000076CCCCCCCCCC0060301800007CC6;
defparam RAM3_inst.INIT_0E = 256'h0000000000007C00386C6C380000000000007E003E6C6C3C0000C6C6CEDEFEF6;
defparam RAM3_inst.INIT_0F = 256'h0606FE0000000000000000C0C0C0FE000000000000007CC6C660303000303000;

defparam RAM4_inst.INIT_00 = 256'h0018180006063E9ECE6630D8CCC6C0C02E2A2EA86E2010E8444240C000000006;
defparam RAM4_inst.INIT_01 = 256'h00000000D86C366CD800000000000000366CD86C360000000000183C3C3C1818;
defparam RAM4_inst.INIT_02 = 256'hDD77DD77DD77DD7755AA55AA55AA55AA55AA55AA114411441144114411441144;
defparam RAM4_inst.INIT_03 = 256'h181818181818181818F8181818181818181818181818181818181818DD77DD77;
defparam RAM4_inst.INIT_04 = 256'h3636363636FE0000000000003636363636F63636363636361818181818F818F8;
defparam RAM4_inst.INIT_05 = 256'h36363636363636363636363636F606F6363636361818181818F818F800000000;
defparam RAM4_inst.INIT_06 = 256'h363636360000000000FE06F6363636363636363636F606FE0000000036363636;
defparam RAM4_inst.INIT_07 = 256'h1818181818F80000000000000000000000F818F8181818180000000000FE3636;
defparam RAM4_inst.INIT_08 = 256'h18FF0000000000000000000000FF18181818181800000000001F181818181818;
defparam RAM4_inst.INIT_09 = 256'h181818180000000000FF00000000000018181818181F18181818181818181818;
defparam RAM4_inst.INIT_0A = 256'h36363636363736363636363618181818181F181F181818181818181818FF1818;
defparam RAM4_inst.INIT_0B = 256'h00FF00F736363636363636363637303F0000000000000000003F303736363636;
defparam RAM4_inst.INIT_0C = 256'h000000003636363636373037363636363636363636F700FF0000000000000000;
defparam RAM4_inst.INIT_0D = 256'h0000000000FF00FF181818183636363636F700F7363636360000000000FF00FF;
defparam RAM4_inst.INIT_0E = 256'h36FF0000000000001818181818FF00FF000000000000000000FF363636363636;
defparam RAM4_inst.INIT_0F = 256'h0000000000000000001F181F1818181800000000003F36363636363636363636;

defparam RAM5_inst.INIT_00 = 256'h3636363636FF36363636363636363636363F00000000000018181818181F181F;
defparam RAM5_inst.INIT_01 = 256'h181F0000000000000000000000F81818181818181818181818FF18FF18181818;
defparam RAM5_inst.INIT_02 = 256'hF0F0F0F0FFFFFFFFFFFF000000000000FFFFFFFFFFFFFFFFFFFFFFFF18181818;
defparam RAM5_inst.INIT_03 = 256'h000000000000FFFFFFFFFFFF0F0F0F0F0F0F0F0F0F0F0F0FF0F0F0F0F0F0F0F0;
defparam RAM5_inst.INIT_04 = 256'hC0C0C0C0C6C6FE0040C0C0FCC6C6FCC67C000000000076DCD8D8DC7600000000;
defparam RAM5_inst.INIT_05 = 256'h000000000000FEC66030183060C6FE0000006C6C6C6C6C6CFE0000000000C0C0;
defparam RAM5_inst.INIT_06 = 256'h00001818181818DC7600000000C060607C66666666000000000070D8D8D8D87E;
defparam RAM5_inst.INIT_07 = 256'h6C6CC6C6C66C38000000386CC6C6FEC6C66C380000007E183C6666663C187E00;
defparam RAM5_inst.INIT_08 = 256'h7E060300000000007EDBDB7E0000000000003C6666663E0C18301E000000EE6C;
defparam RAM5_inst.INIT_09 = 256'h0000C6C6C6C6C6C6C67C000000001C3060607C6060301C000000C0607EF3DBDB;
defparam RAM5_inst.INIT_0A = 256'h30180C060C1830000000FF000018187E18180000000000FE0000FE0000FE0000;
defparam RAM5_inst.INIT_0B = 256'h1818181818181818181818181B1B0E0000007E000C18306030180C0000007E00;
defparam RAM5_inst.INIT_0C = 256'h00000000DC7600DC760000000000001818007E0018180000000070D8D8181818;
defparam RAM5_inst.INIT_0D = 256'h00180000000000000000000000181800000000000000000000000000386C6C38;
defparam RAM5_inst.INIT_0E = 256'h6030D8700000000000006C6C6C6C6CD800001C3C6CEC0C0C0C0C0C0F00000000;
defparam RAM5_inst.INIT_0F = 256'h0000000000000000000000000000007C7C7C7C7C7C000000000000000000F8C8;




/*
//*****************************************************************
//    8 x 8 font keszlet 
//*****************************************************************
defparam RAM0_inst.INIT_00 = 256'h0010387CFEFEFE6C7EFFE7C3FFDBFF7E7E8199BD81A5817E0000000000000000;
defparam RAM0_inst.INIT_01 = 256'h0000183C3C1800007C387CFE7C3810107C387CFEFE387C380010387CFE7C3810;
defparam RAM0_inst.INIT_02 = 256'h78CCCCCC7D0F070FFFC399BDBD99C3FF003C664242663C00FFFFE7C3C3E7FFFF;
defparam RAM0_inst.INIT_03 = 256'h995A3CE7E73C5A99C0E66763637F637FE0F07030303F333F187E183C6666663C;
defparam RAM0_inst.INIT_04 = 256'h0066006666666666183C7E18187E3C1800020E3EFE3E0E020080E0F8FEF8E080;
defparam RAM0_inst.INIT_05 = 256'hFF183C7E187E3C18007E7E7E0000000078CC386C6C38633E001B1B1B7BDBDB7F;
defparam RAM0_inst.INIT_06 = 256'h00003060FE6030000000180CFE0C180000183C7E1818181800181818187E3C18;
defparam RAM0_inst.INIT_07 = 256'h0000183C7EFFFF000000FFFF7E3C180000002466FF6624000000FEC0C0C00000;
defparam RAM0_inst.INIT_08 = 256'h006C6CFE6CFE6C6C00000000006C6C6C00300030787878300000000000000000;
defparam RAM0_inst.INIT_09 = 256'h0000000000C060600076CCDC76386C3800C6663018CCC6000030F80C78C07C30;
defparam RAM0_inst.INIT_0A = 256'h00003030FC3030000000663CFF3C660000603018181830600018306060603018;
defparam RAM0_inst.INIT_0B = 256'h0080C06030180C06003030000000000000000000FC0000006030300000000000;
defparam RAM0_inst.INIT_0C = 256'h0078CC0C380CCC7800FCCC60380CCC7800FC303030307030007CE6F6DECEC67C;
defparam RAM0_inst.INIT_0D = 256'h00303030180CCCFC0078CCCCF8C060380078CC0C0CF8C0FC001E0CFECC6C3C1C;
defparam RAM0_inst.INIT_0E = 256'h603030000030300000303000003030000070180C7CCCCC780078CCCC78CCCC78;
defparam RAM0_inst.INIT_0F = 256'h00300030180CCC78006030180C1830600000FC0000FC000000183060C0603018;

defparam RAM1_inst.INIT_00 = 256'h003C66C0C0C0663C00FC66667C6666FC00CCCCFCCCCC78300078C0DEDEDEC67C;
defparam RAM1_inst.INIT_01 = 256'h003E66CEC0C0663C006060607860607E007E60607860607E00F86C6666666CF8;
defparam RAM1_inst.INIT_02 = 256'h00E6666C786C66E60078CCCC0C0C0C1E007830303030307800CCCCCCFCCCCCCC;
defparam RAM1_inst.INIT_03 = 256'h00386CC6C6C66C3800C6C6CEDEF6E6C600C6C6D6FEFEEEC6007E606060606060;
defparam RAM1_inst.INIT_04 = 256'h0078CC1C70E0CC7800E6666C7C6666FC001C78DCCCCCCC7800F060607C6666FC;
defparam RAM1_inst.INIT_05 = 256'h00C6EEFED6C6C6C6003078CCCCCCCCCC00FCCCCCCCCCCCCC00303030303030FC;
defparam RAM1_inst.INIT_06 = 256'h007860606060607800FE6030180C06FE0078303078CCCCCC00C66C38386CC6C6;
defparam RAM1_inst.INIT_07 = 256'hFF0000000000000000000000C66C381000781818181818780002060C183060C0;
defparam RAM1_inst.INIT_08 = 256'h0078CCC0CC78000000DC66667C6060E00076CC7C0C7800000000000000183030;
defparam RAM1_inst.INIT_09 = 256'hF80C7CCCCC76000000F06060F0606C380078C0FCCC7800000076CCCC7C0C0C1C;
defparam RAM1_inst.INIT_0A = 256'h00E66C786C6660E078CCCC0C0C0C000C007830303070003000E66666766C60E0;
defparam RAM1_inst.INIT_0B = 256'h0078CCCCCC78000000CCCCCCCCF8000000C6D6FEFECC00000078303030303070;
defparam RAM1_inst.INIT_0C = 256'h00F80C78C07C000000F0606676DC00001E0C7CCCCC760000F0607C6666DC0000;
defparam RAM1_inst.INIT_0D = 256'h006CFEFED6C60000003078CCCCCC00000076CCCCCCCC000000183430307C3010;
defparam RAM1_inst.INIT_0E = 256'h001C3030E030301C00FC643098FC0000F80C7CCCCCCC000000C66C386CC60000;
defparam RAM1_inst.INIT_0F = 256'h00FEC6C66C381000000000000000DC7600E030301C3030E00018181800181818;

defparam RAM2_inst.INIT_00 = 256'h0181472595888482179557F4376824E207854725978884821795573497888482;
defparam RAM2_inst.INIT_01 = 256'h078447219788848217955774574824E2078547E5376824E2179557F437E884E2;
defparam RAM2_inst.INIT_02 = 256'h018147E5356824E2179557B4B7A8A4A2078547E537E884E21795773437E8A4E2;
defparam RAM2_inst.INIT_03 = 256'h00CCFCCC7800303000C6C6FEC66C38C6874527554F4422E1179557B497B894B2;
defparam RAM2_inst.INIT_04 = 256'h0078CCCC7800CC7800CECCCCFECC6C3E007FCC7F0C7F000000FC607860FC001C;
defparam RAM2_inst.INIT_05 = 256'h007ECCCCCC00E000007ECCCCCC00CC780078CCCC7800E0000078CCCC7800CC00;
defparam RAM2_inst.INIT_06 = 256'h18187EC0C07E18180078CCCCCCCC00CC00183C66663C18C3F80C7CCCCC00CC00;
defparam RAM2_inst.INIT_07 = 256'h70D818183C181B0EC7C6CFC6FACCCCF83030FC30FC78CCCC00FCE660F0646C38;
defparam RAM2_inst.INIT_08 = 256'h007ECCCCCC001C000078CCCC78001C000078303030700038007ECC7C0C78001C;
defparam RAM2_inst.INIT_09 = 256'h00007C00386C6C3800007E003E6C6C3C00CCDCFCECCC00FC00CCCCCCF800F800;
defparam RAM2_inst.INIT_0A = 256'h07844721F74844C200000C0CFC0000000000C0C0FC0000000078CCC060300030;
defparam RAM2_inst.INIT_0B = 256'h0000CC663366CC0000003366CC663300001818181800181803CF6F37DBCCC6C3;
defparam RAM2_inst.INIT_0C = 256'h1818181818181818EEDB77DBEEDB77DBAA55AA55AA55AA558822882288228822;
defparam RAM2_inst.INIT_0D = 256'h363636FE00000000363636F636363636181818F818F81818181818F818181818;
defparam RAM2_inst.INIT_0E = 256'h363636F606FE00003636363636363636363636F606F63636181818F818F80000;
defparam RAM2_inst.INIT_0F = 256'h181818F800000000000000F818F81818000000FE36363636000000FE06F63636;

defparam RAM3_inst.INIT_00 = 256'h1818181F18181818181818FF00000000000000FF181818180000001F18181818;
defparam RAM3_inst.INIT_01 = 256'h36363637363636361818181F181F1818181818FF18181818000000FF00000000;
defparam RAM3_inst.INIT_02 = 256'h363636F700FF0000000000FF00F7363636363637303F00000000003F30373636;
defparam RAM3_inst.INIT_03 = 256'h000000FF00FF1818363636F700F73636000000FF00FF00003636363730373636;
defparam RAM3_inst.INIT_04 = 256'h0000003F36363636363636FF00000000181818FF00FF0000000000FF36363636;
defparam RAM3_inst.INIT_05 = 256'h363636FF363636363636363F000000001818181F181F00000000001F181F1818;
defparam RAM3_inst.INIT_06 = 256'hFFFFFFFFFFFFFFFF1818181F00000000000000F818181818181818FF18FF1818;
defparam RAM3_inst.INIT_07 = 256'h00000000FFFFFFFF0F0F0F0F0F0F0F0FF0F0F0F0F0F0F0F0FFFFFFFF00000000;
defparam RAM3_inst.INIT_08 = 256'h006C6C6C6C6CFE0000C0C0C0C0CCFC00C0C0F8CCF8CC78000076DCC8DC760000;
defparam RAM3_inst.INIT_09 = 256'h0018181818DC7600C0607C66666666000070D8D8D87E000000FCCC603060CCFC;
defparam RAM3_inst.INIT_0A = 256'h0078CCCC7C18301C00EE6C6CC6C66C3800386CC6FEC66C38FC3078CCCC7830FC;
defparam RAM3_inst.INIT_0B = 256'h00CCCCCCCCCCCC78003860C0F8C06038C0607EDBDB7E0C0600007EDBDB7E0000;
defparam RAM3_inst.INIT_0C = 256'h00FC00183060301800FC00603018306000FC003030FC30300000FC00FC00FC00;
defparam RAM3_inst.INIT_0D = 256'h0000DC7600DC760000303000FC00303070D8D8181818181818181818181B1B0E;
defparam RAM3_inst.INIT_0E = 256'h1C3C6CEC0C0C0C0F0000001800000000000000181800000000000000386C6C38;
defparam RAM3_inst.INIT_0F = 256'h000000000000000000003C3C3C3C000000000078603018700000006C6C6C6C78;
*/


/***********************************************************\
* 	A belso 50MHz - es orajel eloallitasa a 100 MHz - bol    *
*  az FPGA-n beluli DLL alkalmazasaval.                     *                                           
\***********************************************************/

	IBUFG	ibosc(.I(CLK_100MHz), .O(ickosc));
	CLKDLLHF #(
		.STARTUP_WAIT("TRUE"),
		.CLKDV_DIVIDE(1.5),		// 1.5 eseten 66MHz lesz az orajel :-)
		.DUTY_CYCLE_CORRECTION("TRUE")
	) dll_int (
		.CLKIN(ickosc),
		.CLKFB(CLK100),
		.CLK0(ck100),
		.CLKDV(ck50)
	);
	BUFG ob100(.I(ck100), .O(CLK100));
	BUFG  ob50(.I(ck50), .O(CLK_50MHz));

//assign CLK_50MHz = CLK_100MHz;
	
// RAM adatbusz meghajtasa a ram_write_data regiszterbol 
// ha a WE es a CE0 aktiv "L" szintu kulonben (TS)
// A WE es a CE0 labak helyett a ramdbe regiszter van hasznalva
// az egyseges kimeneti idozitesek miatt!
// Minden ram-ba irando adatot a ram_write_data regiszterbe kell tolteni.
assign RAM_DATA = ( rambde == 0 /*!nRAM_WE && !nRAM_CE0*/ ) ? ram_write_data : 8'hzz;

//------------------------------------------------------------------
// A belso regiszterek kikuldese CPU fele meno adatbuszra ha az RD jel 
// aktiv "L" szintu es az WR jel "H" szintu kulonbenm (TS)
// A2..A0 CPU_ADDRESS-tol fuggoen kozvetlenul 7 regiszter erheto el.
// Sajnos csak a 0. es az 1. cimen levo reg olvasasa fert bele az FPGA-ba 
// igy a tobbi regiszter csak irhato lett. :-(
//
// addr 0	CPU <--- cpu_read_data regiszter
// addr 1   CPU <--- cpu_cgram_data regiszter
// addr 2   nc.
// addr 3   nc.
//	addr 4 	nc.
//	addr 5	nc.
// addr 6	nc.
//------------------------------------------------------------------
assign CPU_DATA = ( nCPU_RD == 0 && nCPU_WR == 1 && CPU_ADDRESS == 0 ) ? cpu_read_data :
						( nCPU_RD == 0 && nCPU_WR == 1 && CPU_ADDRESS == 1 ) ? cpu_cgram_data: 8'hzz; 						
//------------------------------------------------------------------
// CPU felol jovo adat betoltese a belso regiszterekbe a WR jel
// lefuto ele utan ha az RD jel "H" szintu.
// A2..A0 CPU_ADDRESS-tol fuggoen kozvetlenul 7 regiszter erheto el.
// addr 0	CPU ---> cpu_write_data 			RAM-ba irando tadatregiszter 
// addr 1   CPU ---> cpu_ram_address[7:0] 	RAM cim also  byte  A7...A0
// addr 2   CPU ---> cpu_ram_address[15:8]	RAM cim felso byte A15...A8
// addr 3   CPU ---> 1 bit-es RAM lapozo regiszterek
//
//				 0  0  0  0  0  0  0  0 <---  Bekapcsolaskor alapallapot 
//				b7 b6 b5 b4 b3 b2 b1 b0 
//				 |  |  |  |  |  |  |  |
//				 |  |  |	 |  |  |  |   ---->	cpu_ram_A16
//				 |	 |  |  |  |  |   ------->	layer1_A16
//				 |	 |  |  |	 |   ---------->	layer2_A16
//				 |  |  |  |   ------------->  layer3_A16
//				 |  |  |   ---------------->	layer4_A16
//				 |	 |   ------------------->  layer5_A16
//				 |	  ---------------------->  layer6_A16
//				  ------------------------->	nc.
//
//
//	addr 4 	CPU ---> cpu_car_attr automatikus karakter attributum irashoz 
//
//				 0  0  0  0  0  0  0  0 <---  Bekapcsolaskor alapallapot 
//				b7 b6 b5 b4 b3 b2 b1 b0 --->	car_attr
//			    |  |  |  |  |  |  ---
//			  	 |  |  |  |  |  |   | 
//				 |  |  |  |  |  |   | 
//			    |  |  |  |  |  |    ------>	fenyero 0...3
//				 |  |	 |  |  |   ---------->	atlatszosag. 0=igen, 1=nem 
//			    |  |  |  |   ------------->  extra ures sor atlatszosag 0=igen, 1=nem 
//			    |  |	 |	  ---------------->	inverz 0=nem, 1=igen
//				 |	 |	  ------------------->  inverz villogas 0=nem, 1=igen			
//				 |	  ---------------------->  villogas 0=igen, 1=nem
//				  ------------------------->  a 2. karakter genarator kivalasztasa
//
//
// addr 5	CPU --- >  layer_enable[6:0] <= CPU_DATA[6:0] Display es layer engedelyezo/tilto regiszter irasa
//							  auto_increment    <= CPU_DATA[7]   Ezt adja hozza ram cimehez, ha a cpu CPU irta vagy olvasta a RAM-ot.
//
//				 0  0  0  0  0  0  0  0 <---  Bekapcsolaskor alapallapot 
//				b7 b6 b5 b4 b3 b2 b1 b0 
//			  	 |  |  |  |  |  |  |  |
//			  	 |  |  |	 |  |  |  |   ---->	layer_enable[0] Layer 1 ( 1bit/pixel )
//				 |	 |  |  |  |  |   ------->	layer_enable[1] Layer 2 ( 1bit/pixel )
//		    	 |	 |  |  |	 |   ---------->	layer_enable[2] Layer 3 ( 1bit/pixel )
//			    |  |  |  |   ------------->  layer_enable[3] Layer 4 ( 1bit/pixel )
//			    |  |  |   ---------------->	layer_enable[4] Layer 5 ( 2bit/pixel )
//				 |	 |   ------------------->  layer_enable[5] Layer 6 ( karakteres )
//				 |	  ---------------------->  layer_enable[6] Display off/on 0 = tiltas, 1 = engedelyezes
//				  ------------------------->	auto_increment  Automatikus cimszamlalo noveles engedelyezes
//
// addr 6	CPU --->	layer_grayscale layer 1...4 fenyerobeallitasok 
//												 ( 0 ertek a legkissebb a 3-as legnagyobb fenyero )
//
//				 0  0  0  0  0  0  0  0 <---  Bekapcsolaskor alapallapot 
//				b7 b6 b5 b4 b3 b2 b1 b0 --->	layer_grayscale regiszter
//			    ---   ---   ---   ---
//			  	  |     |     |     | 
//				  |     |     |     | 
//			     |     |     |      ------>	Layer 1
//				  |	  |      ------------>	Layer 2
//			     |      ------------------>  Layer 3
//			      ------------------------>	Layer 4
//
//	addr 7	CPU --->	car_size[2:0]		<= CPU_DATA[2:0]	A karakter magassagot allitja be
//							car_gen_size[2:0]	<= CPU_DATA[5:3]	A kar.gen. karakter magassagat allitja be
//							ext_car_gen			<= CPU_DATA[6]		Az 1. karaktergenerator helyet allitja be
//							write_car_attr		<= CPU_DATA[7]		Ha 1, akkor a ram irasakor ket ram ciklus
//																			lessz es a masodikban 1200 byte-al magasabb
//																			cimre a cpu_car_attr lessz beirva
//
//				 0  0  0  0  0  0  0  0 <---  Bekapcsolaskor alapallapot ( 8*8  -as belso kar. gen. )
//				 0  0  0  1  0  1  0  0 <---  Bekapcsolaskor alapallapot ( 8*12 -as belso kar. gen. )
//				b7 b6 b5 b4 b3 b2 b1 b0   
//  		    |  | -------- --------
//	   	  	 |  |    |        |     
//		   	 |  |    |        |      
//		       |  |    |         --------->	car_size[2:0] 	   
//	          |  |     ------------------> car_gen_size[2:0] 
//				 |   ----------------------->	ext_car_gen  		
//            --------------------------> write_car_attr		
//
//------------------------------------------------------------------

					
						
always @ ( posedge CLK_50MHz ) begin



	// A belso karaktergenerator CPU fele torteno olvasasa
	if ( cpu_cgram_mem_req == 1 ) begin
		case ( cpu_cgram_cikl )
			0: 
				begin 
					CPU_CGRAM_ADDR[8:0] <= cpu_ram_address [8:0];
					CPU_CGRAM_CLK 		  <= 0;
					CPU_CGRAM_EN		  <= 1;
					cpu_cgram_cikl 	  <= 1;
				end
			1:	
				begin
					CPU_CGRAM_CLK 	<= 1;
					cpu_cgram_cikl <= 2;
				end
			2:
				begin
					case ( cpu_ram_address[11:9] )
						3'b000:	cpu_cgram_data[7:0] = CPU_CGRAM_DATA0[7:0];
						3'b001:	cpu_cgram_data[7:0] = CPU_CGRAM_DATA1[7:0];
						3'b010:	cpu_cgram_data[7:0] = CPU_CGRAM_DATA2[7:0];
						3'b011:	cpu_cgram_data[7:0] = CPU_CGRAM_DATA3[7:0];
						3'b100:	cpu_cgram_data[7:0] = CPU_CGRAM_DATA4[7:0];
						3'b101:	cpu_cgram_data[7:0] = CPU_CGRAM_DATA5[7:0];
					endcase
					cpu_cgram_cikl 	<= 0;
					cpu_cgram_mem_req <= 0;
					CPU_CGRAM_CLK 		<= 0;
					CPU_CGRAM_EN		<= 0;
				end
		endcase
	end




	// RAM olvasas kesz
	PROBA = mem_req[6] ;

	// **************************************************
	// *** A command regisztertol fuggoen CPU olvasas ***
	// **************************************************
	// Ha az nCPU_WR nem aktiv, akkor vizsgalja hogy akar-e olvasni a CPU
	if ( nCPU_WR == 1 && ncpu_rd_edge == Lefuto_el ) begin
		// CPU RAM olvasas
		if ( CPU_ADDRESS == 0 ) mem_req[0] <= 1;		 // RAM olvasas hozzaferes igeny jelzes

		// CPU a karaktergeneratort olvassa
		if ( CPU_ADDRESS == 1 ) cpu_cgram_mem_req <= 1;	 // CGRAM olvasas hozzaferes igeny jelzes 

	end

	
	// *************************************************
	// ***  A command regisztertol fuggoen CPU iras  ***
	// *************************************************
	// Ha az nCPU_RD nem aktiv, akkor vizsgalja hogy akar-e irni a CPU
	if ( nCPU_RD == 1 && ncpu_wr_edge == Lefuto_el ) begin	
		if ( CPU_ADDRESS == 0 ) begin 
			// CPU RAM iras
			cpu_write_data <= CPU_DATA; 	// CPU adat a cpu_write_data regiszterbe
			mem_req[1] <= 1;				 	// RAM irass hozzaferes igeny jelzes irashoz
		end

		//	RAM cimzo regiszter also byte irasa
		if ( CPU_ADDRESS == 1 ) cpu_ram_address[7:0]  <= CPU_DATA;
	
		//	RAM cimzo regiszter felso byte irasa
		if ( CPU_ADDRESS == 2 ) cpu_ram_address[15:8] <= CPU_DATA;

		if ( CPU_ADDRESS == 3 )	begin 	// RAM lapozo regiszter
			cpu_ram_A16			 <= CPU_DATA[0];
			layer1_A16			 <= CPU_DATA[1];	
			layer2_A16			 <= CPU_DATA[2];	
			layer3_A16			 <= CPU_DATA[3];	
			layer4_A16			 <= CPU_DATA[4];	
			layer5_A16			 <= CPU_DATA[5];	
			layer6_A16			 <= CPU_DATA[6];
			display_inv			 <= CPU_DATA[7];
		end
		if ( CPU_ADDRESS == 4 )	cpu_car_attr	  	<= CPU_DATA;	// CPU karakter attributum

		if ( CPU_ADDRESS == 5 ) begin 
			layer_enable[6:0]	<= CPU_DATA[6:0];		// Display es layer engedelyezo/tilto regiszter irasa
			auto_increment_en <= CPU_DATA[7];		// Ezt adja hozza ram cimehez, ha a cpu CPU irta vagy olvasta a RAM-ot.
		end
		
		if ( CPU_ADDRESS == 6 )	layer_grayscale   <= CPU_DATA;	// Layer1..4 fenyero regiszter irasa
				

		if ( CPU_ADDRESS == 7 ) begin
			car_size[2:0]		<= CPU_DATA[2:0];	// A karakter magassagot allitja be
			car_gen_size[2:0]	<= CPU_DATA[5:3];	// A karaktergenerator magassagat allitja be
			ext_car_gen			<= CPU_DATA[6];	// Az 1. karaktergenerator helyet allitja be
			write_car_attr		<= CPU_DATA[7];	// Ha 1, akkor a ram irasakor ket ram ciklus
															// lessz es a masodikban 1200 byte-al magasabb
															// cimre a cpu_car_attr lessz beirva
		end
	end


/********************************************************************\
* 	Eldetektalasok                                                    *
* 	                                                                  *
* 	00 = "L" szint, 01 = Felfuto el, 11 = "H" szint, 10 = Lefuto el   *
* 	                                                                  *
\********************************************************************/

	// CPU RD jel az eldetektalashoz egy 2bit-es shiftregiszterbe
	ncpu_rd_edge[0] <= nCPU_RD;
	ncpu_rd_edge[1] <= ncpu_rd_edge[0];

	// CPU WR jel az eldetektalashoz a 2bit-es shiftregiszterbe
	ncpu_wr_edge[0] <= nCPU_WR;
	ncpu_wr_edge[1] <= ncpu_wr_edge[0];

//------------------------------------------------------------------



	// Az LCD shift clock jel felfuto elenel minden engedelyezett layer-hez 
	// tartozo memoria hozzaferest kero bit-et 1-be irja.
	if ( lcd_cp_0_felfutas == 1 ) begin 
		nRAM_CE0 <= 0;
		lcd_layer_data_puffer <= 0;
		if ( layer1_enable == 1 ) mem_req[2] <= 1;	// Layer 1 RAM memoria olvasas keres jelzes
		if ( layer2_enable == 1 ) mem_req[3] <= 1;	// Layer 2 RAM memoria olvasas keres jelzes
		if ( layer3_enable == 1 ) mem_req[4] <= 1;	// Layer 3 RAM memoria olvasas keres jelzes
		if ( layer4_enable == 1 ) mem_req[5] <= 1;	// Layer 4 RAM memoria olvasas keres jelzes
		if ( layer5_enable == 1 ) mem_req[6] <= 1;	// Layer 5 RAM memoria olvasas keres jelzes
		if ( layer6_enable == 1 ) mem_req[7] <= 1;	// Layer 6 RAM memoria olvasas keres jelzes

		if ( lcd_x_ct_max == 1 && lcd_y_ct_max == 1 ) begin 
			// Layer engedelyezesek layer_enable regiszter bitjeitol fuggoen
			layer1_enable <= layer_enable[0];
			layer2_enable <= layer_enable[1];
			layer3_enable <= layer_enable[2];
			layer4_enable <= layer_enable[3];
			layer5_enable <= layer_enable[4];
			layer6_enable <= layer_enable[5];
			// Minden kepvaltas alkalmaval novekszik. A fenyerobeallitasoknal van hasznalva.
			frame_ct <= frame_ct + 1;
		end
	end	

	// Prioritasba rendezve a ram hozzaferes engedelyezese
	// Sorrendben 	legmagasabb 	: 	CPU olvasas
	//											CPU iras
	//											Layer 1
	//											Layer 2
	//											Layer 3
	//											Layer 4
	//											Layer 5
	//					legalacsonyabb	:	Layer 6

	if ( ram_cikl == 0 ) ram_cikl <= mem_req_start;


// *** EGYSZERUSITESEK ***
//
	// A lookup tabla also ket cime mindig ugyanaz ezert azt
	// csak egyszer irom le ( -1db slice )
	look_up_table_addr[1:0] = yx[1:0];
	

	case ( ram_cikl )
		1: // CPU olvas a RAM-bol
			begin
				RAM_ADDRESS 	<= { cpu_ram_A16, cpu_ram_address }; 
				rambde   <= 1;
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				ram_cikl <= 2;	
			end
		2:
			begin
				mem_req[0] 	<= 0;
				ram_cikl 	<= 3;
			end		
		3:
			begin
				// Ha az automatikus leptetes engedelyezett, akkor leptetem a CPU cimszamlalot
				cpu_ram_address <= cpu_ram_address + auto_increment_en; 
				cpu_read_data = RAM_DATA;
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				ram_cikl <= mem_req_start;
			end
		4:	// CPU ir a video RAM-ba
			begin
				RAM_ADDRESS[16] <= cpu_ram_A16;
				if ( cpu_byte_ct == 0 ) begin
					RAM_ADDRESS[15:0]	<= cpu_ram_address[15:0]; 
					ram_write_data <= cpu_write_data;
				end
				else begin
					RAM_ADDRESS[15:0]	 <= cpu_ram_address[15:0] + 1200; 
					ram_write_data  <= cpu_car_attr;
				end
				rambde   <= 1;
				nRAM_WE	<= 1;
				nRAM_OE  <= 1;
				ram_cikl <= 5;	
				if ( ( cpu_ram_address > 57599 ) && ( cpu_ram_address < 60001  ) ) attr_write_en <= 1; else attr_write_en <= 0;
			end
		5:
			begin
				mem_req[1] <= ( ~cpu_byte_ct & write_car_attr & attr_write_en );
				nRAM_WE	<= 0;
				rambde   <= 0;
				nRAM_OE  <= 1;
				ram_cikl <= 6;	
			end
		6:
			begin
				nRAM_WE	<= 1;
				rambde   <= 1;
				nRAM_OE  <= 1;
				if ( cpu_byte_ct == 0 && write_car_attr == 1 && attr_write_en == 1 ) begin 
					ram_cikl <= 4; 
					cpu_byte_ct <= 1;
				end 
				else begin 
					cpu_byte_ct <= 0;
					ram_cikl <= mem_req_start;
					// Ha az automatikus leptetes engedelyezett, akkor leptetem a CPU cimszamlalot
					cpu_ram_address <= cpu_ram_address + auto_increment_en; 
				end
			end
		7:	// A display terito olvassa a layer1-et a video RAM-bol
			begin
				RAM_ADDRESS[16]	<= layer1_A16;
				RAM_ADDRESS[15:0] <= layer1_start + display_addr; 
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				ram_cikl <= 8;	
				look_up_table_addr[3:2] = layer_grayscale[1:0];
			end
		8:	
			begin
				ram_cikl <= 9;	
				mem_req[2] <= 0;
			end
		9:	
			begin
				lcd_layer_data_puffer <= lcd_layer_data_puffer | ( RAM_DATA & look_up_table_data );
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				ram_cikl <= mem_req_start;
			end
		10:	// A display terito olvassa a layer2-ot a video RAM-bol
			begin
				RAM_ADDRESS[16]	<= layer2_A16;
				RAM_ADDRESS[15:0] <= layer2_start + display_addr; 
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				look_up_table_addr[3:2] = layer_grayscale[3:2];
				ram_cikl <= 11;	
			end
		11:	
			begin
				ram_cikl <= 12;	
				mem_req[3] <= 0;
			end
		12:	
			begin
				lcd_layer_data_puffer <= lcd_layer_data_puffer | ( RAM_DATA & look_up_table_data );
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				ram_cikl <= mem_req_start;
			end
		13:	// A display terito olvassa a layer3-at a video RAM-bol
			begin
				RAM_ADDRESS[16]	<= layer3_A16;
				RAM_ADDRESS[15:0] <= layer3_start + display_addr; 
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				look_up_table_addr[3:2] = layer_grayscale[5:4];
				ram_cikl <= 14;	
			end
		14:	
			begin
				ram_cikl <= 15;	
				mem_req[4] <= 0;
			end
		15:	
			begin
				lcd_layer_data_puffer <= lcd_layer_data_puffer | ( RAM_DATA & look_up_table_data );
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				ram_cikl <= mem_req_start;
			end
		16:	// A display terito olvassa a layer4-et a video RAM-bol
			begin
				RAM_ADDRESS[16]	<= layer4_A16;
				RAM_ADDRESS[15:0] <= layer4_start + display_addr; 
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				look_up_table_addr[3:2] = layer_grayscale[7:6];
				ram_cikl <= 17;	
			end
		17:	
			begin
				ram_cikl <= 18;	
				mem_req[5] <= 0;
			end
		18:	
			begin
				lcd_layer_data_puffer <= lcd_layer_data_puffer | ( RAM_DATA & look_up_table_data );
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				ram_cikl <= mem_req_start;
			end
		19:	// A display terito olvassa a layer5-ot a video RAM-bol
			begin
				RAM_ADDRESS[16]	<= layer5_A16;
				if ( grayscale_ovf == 0 ) begin 
					RAM_ADDRESS[15:0] <= layer5_start + grayscale_display_addr + grayscale_byte_ct; 
				end
				else begin
					if ( grayscale_byte_ct == 0 ) begin 
						RAM_ADDRESS <= layer5_start + grayscale_display_addr;
					end
					else begin
						RAM_ADDRESS <= layer5_start; 
					end
				end
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				ram_cikl <= 20;	
			end
		20:
			begin
				ram_cikl <= 21;	
				// Csak akkor torli a memoria hozzaferest kero bit-et, ha a masodik byte-ot is felolvasta!
				if ( grayscale_byte_ct == 1 ) mem_req[6] <= 0;
			end
		21:
			begin
			
				rom3_addr[3:0] = { RAM_DATA[7:6], yx[1:0] };
				rom2_addr[3:0] = { RAM_DATA[5:4], yx[1:0] };
				rom1_addr[3:0] = { RAM_DATA[3:2], yx[1:0] };
				rom0_addr[3:0] = { RAM_DATA[1:0], yx[1:0] };

				if ( grayscale_byte_ct == 0 )	begin
					lcd_layer_data_puffer[7:4] <= lcd_layer_data_puffer[7:4] | { rom0_data, rom3_data, rom2_data, rom1_data };
					grayscale_byte_ct <= 1;
				end

				if ( grayscale_byte_ct == 1 )	begin 
					lcd_layer_data_puffer[3:0] <= lcd_layer_data_puffer[3:0] | { rom3_data, rom2_data, rom1_data, rom0_data };
				end
				
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				
				// Ha a masodik byte-ot is felolvasta torli a byte szamlalot es
				// ha van ram keres, akkor azzal folytatja!
				if ( grayscale_byte_ct == 1 ) begin
					ram_cikl <= mem_req_start; 
					grayscale_byte_ct <= 0;
				end 
				else begin
					ram_cikl <= 19;	
				end
			end
		22:	// A display terito olvassa a karakteres layer6-ot a video RAM-bol
				// 3 byte-ot olvas, mert van attributuma is a karaktereknek es a
				// karaktergenerator is a RAM-ban van
			begin
				if ( car_byte_ct[1] == 0 ) begin 
					RAM_ADDRESS[16]	<= layer6_A16;
					if ( car_byte_ct[0] == 0 ) begin
						// Karakter kod cimzese
						RAM_ADDRESS[15:0] <= layer6_start + car_display_addr[10:0]; 
					end
					else begin
						// Karakter attributum cimzese
						RAM_ADDRESS[15:0] <= layer6_start + car_display_addr + 1200; 
					end
				end
				else begin
					// Karaktergenerator cimzese
					// Ha az karakter attributum legfelso bitje 1, akkor 64kbyte-tal
					// feljebb levo karaktergeneratort cimzi, de csak akkor ha a
					// belso karaktergenerator le van tiltva
					RAM_ADDRESS[16]	<= ( car_attr[7] & ext_car_gen ); 
					RAM_ADDRESS[15:0] <= cg_ram_start[15:0] + {4'b0, cgram_addr[11:0] }; 
					CGRAM_ADDR[8:0] <= cgram_addr[8:0]; 
					CGRAM_EN 	<= 1;
				end
				nRAM_WE	<= 1;
				nRAM_OE  <= 0;
				ram_cikl <= 23;
				look_up_table_addr[3:2] = car_attr[1:0];
			end
		23:
			begin
				CGRAM_CLK 	<= 1;
				// Csak akkor torli a memoria hozzaferest kero bit-et, ha a 3. byte-ot is felolvasta!
				if ( car_byte_ct[1] == 1 ) mem_req[7] <= 0;
				ram_cikl <= 24;
			end
		24:
			begin
				if ( car_byte_ct[1] == 0 ) begin
					if ( car_byte_ct[0] == 0) begin
						// Karakter kod beolvasasa
						car_data <= RAM_DATA;
					end
					else begin
						// A karakter attributum beolvasas
						car_attr <= RAM_DATA;
					end
					ram_cikl <= 22;
					car_byte_ct <= car_byte_ct +1;
				end
				else begin 
					// Megvizsgalja hogy a karakterterites a karaktergeneratorban eltarolt 
					// karaktermereten belul van-e es ha nem extra_line regisztert 1-be irja.
					if ( car_line_ct > { 1'b1, car_gen_size } ) begin 
						extra_line 	<= 1; 
						car_data 	<= 0;
					end
					else begin 
						// Ha a belso karaktergenerator tiltva van, akkor mindent
						// a videorambol olvas
						if ( ext_car_gen == 1 ) begin
							car_data 	<= RAM_DATA;
						end
						// Ha a belso karaktergenerator engedelyezve van, akkor a car_attr[7]- nek
						// megfelelo helyrol olvas
						else begin
							// A belso karaktergeneratorbol olvas
							if ( car_attr[7] == 0 ) begin
								case ( cgram_addr[11:9] )
									3'b000:	car_data[7:0] 	<= CGRAM0_DATA[7:0];
									3'b001:	car_data[7:0] 	<= CGRAM1_DATA[7:0];
									3'b010:	car_data[7:0] 	<= CGRAM2_DATA[7:0];
									3'b011:	car_data[7:0] 	<= CGRAM3_DATA[7:0];
									3'b100:	car_data[7:0] 	<= CGRAM4_DATA[7:0];
									3'b101:	car_data[7:0] 	<= CGRAM5_DATA[7:0];
								endcase
							end
							// A kulso karaktergeneratorbol olvas
							else begin
								car_data 	<= RAM_DATA;
							end
						end
						extra_line 	<= 0;
					end
					// Karakteres layer olvasas kesz
					car_read_ok_ct <= 1; 
					car_byte_ct <= 0;
					ram_cikl <= mem_req_start;
				end
				nRAM_WE	<= 1;
				nRAM_OE	<= 1;
				CGRAM_CLK 	<= 0;
				CGRAM_EN 	<= 0;
			end
//									b7 b6 b5 b4 b3 b2 b1 b0  -->	car_attr
//								    |  |  |  |  |  |  ---
//								  	 |  |  |  |  |  |   | 
//									 |  |  |  |  |  |   | 
//								    |  |  |  |  |  |    ------>	fenyero 0...3
//									 |  |	 |  |  |   ---------->	atlatszosag. 0=igen, 1=nem 
//								    |  |  |  |   ------------->  extra ures sor atlatszosag 0=igen, 1=nem 
//								    |  |	 |	  ---------------->	inverz 0=nem, 1=igen
//									 |	 |	  ------------------->  inverz villogas 0=nem, 1=igen			
//									 |	  ---------------------->  villogas 0=igen, 1=nem
//									  ------------------------->  a 2. karakter genarator kivalasztasa
			
	endcase
	
	// ASZINKRON-RA ATIRVA HOGY KISSEBB LEGYEN A CHIP !!!
	// !!!! cgram_addr <= ( car_data * { 1'b1, car_gen_size } ) + car_line_ct - 1; 
	cgram_addr <= ( car_data * { 1'b1, car_gen_size } ) + car_line_ct - 1; 
	
	case ( car_read_ok_ct )
		1:
			begin
				// Statikusan invertalja a karaktert
				if ( car_attr[4] == 1 ) car_data <= ~car_data;
				car_read_ok_ct <= car_read_ok_ct + 1;
			end
		2:
			begin
				if ( car_attr[5] == 1 ) begin
					// Periodikusan invertalja a karaktert ha engedelyezve van
					if ( frame_ct[5] == 1 ) car_data <= ~car_data;
				end
				car_read_ok_ct <= car_read_ok_ct + 1;
			end
		3:
			begin
				// Ha engedelyezve van a villogas akkor a frame_ct[5] utemeben
				// villogtatja a karaktert.
				if ( !car_attr[6] == 1 || !frame_ct[5] == 1 ) begin
				// Nincs villogas
					// Extra sorok a teritesnel
					if ( extra_line == 1 ) begin
						// Ha NEM atlatszo az extra sor akkor teriti, egyebkent nem csinal semmit
						if ( car_attr[3] == 1 ) lcd_layer_data_puffer <= ( car_data & look_up_table_data );
					end
					// Nincs extra sor a teritesnel
					else begin
						// Ha NEM atlatszo a karakter, akkor mindent felulir
						if ( car_attr[2] == 1 ) begin
							lcd_layer_data_puffer <= ( car_data & look_up_table_data );
						end
						// Ha atlatszo, akkor ossze mixeli a tobbivel
						else begin
							lcd_layer_data_puffer <= lcd_layer_data_puffer | ( car_data & look_up_table_data );
						end
					end
				end
				car_read_ok_ct <= 0;
			end
	endcase



	// LCD DATA meghajtasa
	// Az LCD shift clock jel felfuto elenel az osszes beolvasott layert a megfelelo 
	// logikai kapcsolattal kiirja az LCD adatbuszra. Ha valamit nem sikerult idoben 
	// kiolvasni a video ram-bol, akkor annak a helyere 0 ertek kerul.
	// Ez akkor fordulhat elo ha tul gyorsan irja vagy olvassa a CPU a video RAM-ot.
	// A 4bit-es LCD busz miatt ket reszletben kuldi ki a RAM-bol kiolvasott adatokat
	// az lcd_x_ct 0. bitjetol fuggoen. Ha 0 akkor a felso 4 bit, ha 1 akkor az also 4 bit.
	if ( lcd_cp_0_felfutas == 1 ) begin
			if ( display_inv == 0 ) begin 
				LCD_DATA[3:0] 		<= lcd_layer_data_puffer[7:4]; 
			end
			else begin 
				LCD_DATA[3:0] 		<= ~lcd_layer_data_puffer[7:4]; 
			end
			lcd_out_puff[3:0] <= lcd_layer_data_puffer[3:0];
	end
	
	if ( lcd_cp_1_felfutas == 1 ) begin 
			if ( display_inv == 0 ) begin 
				LCD_DATA[3:0] <= lcd_out_puff[3:0]; 
			end
			else begin
				LCD_DATA[3:0] <= ~lcd_out_puff[3:0]; 
			end
	end	  
end





/********************************************************************\
* 													                              *
* 	               LCD szinkron jelek eloallitasa       				   *	
* 													                              *
\********************************************************************/
/********************************************************************\
* 	 LCD_CP    *                                       				   *	
***************     >|       |< 740ns					                  *
* 		                ___     ___     _             ___	   __       *
*         	 CP     _|   |___|   |___|  ...... |___|   |___|         *
* 													                              *
* 	LCD_CP Shift clock	1.388MHz (50MHz/38) 72Hz-es frissiteshez    *
\********************************************************************/
always @ ( posedge CLK_50MHz ) begin
	lcd_cp_0_felfutas <= 0;
	lcd_cp_1_felfutas <= 0;
	cp_clk_div 			<= cp_clk_div + 1;
	if ( cp_clk_div == 22 ) begin // 66MHz-nel FRAME 75Hz
	//if ( cp_clk_div == 18 ) begin // 50MHz-nel FRAME 72Hz
		LCD_CP  		<= LCD_CP + 1;
		cp_clk_div	<= 0;
		if ( LCD_CP == 0 ) begin 
			// Az LCD_CP-nek az elso es a masodik felfutasat jelzi az
			// lcd_cp_0_felfutas es az lcd_cp_1_felfutas
			if ( lcd_cp_ct == 1 ) begin 
				lcd_cp_1_felfutas <= 1; 
			end 
			else begin 
				lcd_cp_0_felfutas <= 1;
				// A terites vegen van, akkor minden szamlaloba
				// betolti a kezdoerteket
				if ( lcd_x_ct_max == 1 && lcd_y_ct_max == 1 ) begin 
					display_addr <= 1;
					grayscale_display_addr <= 3;
					car_display_addr <= 1;
					car_x_ct <= 1;
					car_line_ct <= 1;
				end
				else begin
					// Az 1 bit/pixel-es ram cim novelese
					if ( display_addr == 9599 ) begin 
						display_addr <= 0; 
					end 
					else begin 
						display_addr <= display_addr + 1;
					end
					// Az 2 bit/pixel-es ram cim novelese
					if ( grayscale_display_addr == 19199 ) begin
						grayscale_display_addr <= 1;
					end
					else begin
						grayscale_display_addr <= grayscale_display_addr + 2;
					end
					
					// A karekteres layer-hez tartozo szamlalok novelese
					if ( car_x_ct == 39 ) begin
						// Ha a sor vegere ert, akkor novelem karakteren beluli sorszamlalot
						// es nullazom az oszlop szamlalot.
						car_x_ct <= 0;
						if ( car_line_ct == { 1'b1, car_size } ) begin
							// Ha a karakterterites utolso soraba ert torlom a
							// sorszamlalot es novelem a ram cimet
							car_line_ct <= 1;
							// EGYSZERUSITES KIHAGYVA MERT CSAK NAGYOBB LESSZ A CHIP
							//if ( car_display_addr == 1199 ) begin
							if ( car_display_addr_1199 == 1 ) begin
								car_display_addr <= 0;
							end
							else begin
								car_display_addr <= car_display_addr + 1;
							end
						end
						else begin
							// Amennyiben nem ert a karakterterites utolso soraba
							// novelem a sorszamlalot es csokkentem 39 karakterrel a ram cimet.
							if ( lcd_y_ct_max == 0 ) begin 
								car_line_ct <= car_line_ct + 1;
								car_display_addr <= car_display_addr - 39;
							end
							else begin
								car_line_ct <= 1;
								car_display_addr <= 0;
							end
							
						end
					end
					else begin
						// Ha nem ert a sor sor vegere akkor novelem a ram cimet es
						// az oszlop szamlalot
						car_x_ct <= car_x_ct + 1;
						// EGYSZERUSITES KIHAGYVA MERT CSAK NAGYOBB LESSZ A CHIP
						//if ( car_display_addr == 1199 ) begin
						if ( car_display_addr_1199 == 1 ) begin 
							car_display_addr <= 0;
						end
						else begin
							car_display_addr <= car_display_addr + 1;
						end
					end
				end
			end


			// Az lcd_cp_ct=0 ha a frame kezdete van, egyebkent mindig invertalodik, 
			// ugy hogy minden masodik LCD CP ciklusban 1 lessz.
			if ( lcd_x_ct_max == 1 && lcd_y_ct_max == 1 ) lcd_cp_ct <= 1; else lcd_cp_ct <= lcd_cp_ct + 1;


			// Az lcd sor es oszlop szamlaloinak novelese
			if ( lcd_x_ct_max == 1 ) begin
				lcd_x_ct <= 0;
				if ( lcd_y_ct_max == 1 ) begin 
					lcd_y_ct <= 0;
				end 
				else begin 
					lcd_y_ct <= lcd_y_ct + 1; 
				end
			end
			else begin
				lcd_x_ct <= lcd_x_ct + 1;
			end
			
		end 
	end
	// EGYSZERUSITES KIHAGYVA MERT CSAK NAGYOBB LESSZ A CHIP
	// Ez az egyszerusites ( + 2db slice )
	// Ha a car_display_addr elerte a maximalis erteket,
	// akkor az car_display_addr_1199 -et 1-be irja kulonben 0
	if ( car_display_addr == 1199 ) car_display_addr_1199 <= 1; else car_display_addr_1199 <= 0;

	// Ha az lcd_y_ct elerte a maximalis erteket,
	// akkor az lcd_y_ct_max -ot 1-be irja kulonben 0
	if ( lcd_y_ct == 239 ) lcd_y_ct_max <= 1; else lcd_y_ct_max <= 0;

	// Ha az lcd_x_ct elerte a maximalis erteket,
	// akkor az lcd_x_ct_max -ot 1-be irja kulonben 0
	if ( lcd_x_ct == 79 ) lcd_x_ct_max <= 1; else lcd_x_ct_max <= 0;

end
//------------------------------------------------------------------


/********************************************************************\
* 	 LCD_LOAD	*								    	                        * 	
****************                                                     *
* 	                 >|       |< 740ns					                  *
* 		                ___     ___     _             ___	   __       *
*         	 CP     _|   |___|   |___|  ...... |___|   |___|         *
* 													                              *
* 		 		               ________	        	   ________	            *
* 		 LCD_LOAD     ______|        |___......__|        |____        *
* 													                              *
* 					         >||< 130ns					                     *
* 					                 >| |< 200ns		                     *
* 					          >|        |< 810ns				               *
\********************************************************************/
always @ ( posedge CLK_50MHz ) begin
	// ASZINKRON-RA ATIRVA HOGY KISSEBB LEGYEN A CHIP !!!
	// !!!! if ( cp_clk_div ==  6 && LCD_LOAD == 0 && lcd_x_ct_max == 1 && LCD_CP == 0 ) LCD_LOAD <= 1;
	// !!!! if ( cp_clk_div == 10 && LCD_LOAD == 1 && lcd_x_ct_max == 1 && LCD_CP == 0 ) LCD_LOAD <= 0;
	if ( cp_clk_div ==  6 && LCD_LOAD == 0 && lcd_x_ct_max == 1 && LCD_CP == 0 ) LCD_LOAD = 1;
	if ( cp_clk_div == 10 && LCD_LOAD == 1 && lcd_x_ct_max == 1 && LCD_CP == 0 ) LCD_LOAD = 0;
end
//------------------------------------------------------------------

/********************************************************************\
*	 LCD_FRAME	*    			                        					   *
****************	    						                              *
*	                  >|              |< 59.2us			               *
*		                 ______________		             ________      *
*      LCD_FRAME   ___|              |_______...... __|		         *
*													                              *
*		 		       	  ___	           ___ 	          ___	         *
*		 LCD_LOAD    ___|   |__________|   |___...... __|   |____      *
*													                              *
*				         >|   |< 810ns		    		                     *
*													                              *
\********************************************************************/
always @ ( posedge LCD_LOAD ) begin
//	!!!! if ( lcd_y_ct == 0 && lcd_x_ct_max == 1 ) LCD_FRAME <= 1;
//	!!!! if ( lcd_y_ct == 1 && lcd_x_ct_max == 1 ) LCD_FRAME <= 0;
	if ( lcd_y_ct == 0 && lcd_x_ct_max == 1 ) LCD_FRAME = 1;
	if ( lcd_y_ct == 1 && lcd_x_ct_max == 1 ) LCD_FRAME = 0;
	// display engedelyezes vagy tiltas 
	// ASZINKRON-RA ATIRVA HOGY KISSEBB LEGYEN A CHIP !!!
	// !!!! nLCD_DISP_OFF <= layer_enable[7] ;
	nLCD_DISP_OFF <= layer_enable[6] ;

end
//------------------------------------------------------------------


/********************************************************************\
*	LCD terites sor es oszlop szamlalok	 *  			    					*
*****************************************   									*
*													   									*
*	lcd_x_ct 0...79 ,  lcd_y_ct 0...239					   					*
\********************************************************************/
always @ ( posedge CLK_50MHz ) begin

	// A ROM cimzesek megkavarasahoz hasznalt 2 bit-es regiszter beallitasa
	// Eredetileg az lcd_y_ct es az lcd_x_ct volt hasznalva, de modositani kellett
	// a 2 LCD CP ideig tarto RAM ciklus miatt.
	if ( lcd_cp_0_felfutas == 1 ) begin 
		// ASZINKRON-RA ATIRVA HOGY KISSEBB LEGYEN A CHIP !!!
		// !!!! yx[0] <= yx[0] + 1;
		// !!!! yx[1] <= lcd_y_ct[0];
		yx[0] = yx[0] + 1;
		yx[1] = lcd_y_ct[0];
		// ASZINKRON-RA ATIRVA HOGY KISSEBB LEGYEN A CHIP !!!
		// !!!! if ( grayscale_display_addr == 19199 ) grayscale_ovf <= 1; else grayscale_ovf <= 0;
		if ( grayscale_display_addr == 19199 ) grayscale_ovf = 1; else grayscale_ovf = 0;
	
	end
end
//------------------------------------------------------------------

endmodule


// Look-up tabla az arnyalatokhoz
//
module look_up_table( rom_addr, x, rom_data );
	input 	[3:0]		rom_addr;		// b3 b2 b1 b0
												//	 ---	^	 ^
												//	  |	|	 |
												//	  |	|	  -- lcd_x_ct
												//	  |	 ------ lcd_y_ct
												//	  |
												//		---------- layer_grayscale

	input		[1:0]		x;					// frame_ct

	output	[7:0]		rom_data;		// kimeneti ertek
	reg		[7:0]		rom_data;
	wire		[3:0]		nn;

	assign nn = { rom_addr[3:2], rom_addr[1:0] + x[1:0] };
	
	always @ ( nn ) begin
		case (nn)
			// 1 vilasos, 3 sotet
			 0: rom_data = 8'b00010001;
			 1: rom_data = 8'b10001000;
			 2: rom_data = 8'b01000100;
			 3: rom_data = 8'b00100010;
			// 2 vilagos, 2 sotet
			 4: rom_data = 8'b01010101;
			 5: rom_data = 8'b10101010;
			 6: rom_data = 8'b01010101;
			 7: rom_data = 8'b10101010;
			// 3 vilagos, 1 sotet
			 8: rom_data = 8'b01110111;
			 9: rom_data = 8'b11101110;
			10: rom_data = 8'b11011101;
			11: rom_data = 8'b10111011;
			// 4 vilagos, 0 sotet
			12: rom_data = 8'b11111111;
			13: rom_data = 8'b11111111;
			14: rom_data = 8'b11111111;
			15: rom_data = 8'b11111111;
		endcase
	end
endmodule


module rom3( rom_addr, x, rom_data );
	input 	[3:0]		rom_addr;		// b3 b2 b1 b0
												//	 ---	^	 ^
												//	  |	|	 |
												//	  |	|	  -- lcd_x_ct
												//	  |	 ------ lcd_y_ct
												//	  |
												//		---------- bit_grayscale

	input		[1:0]		x;					// frame_ct

	output				rom_data;		// kimeneti ertek
	reg					rom_data;
	wire		[3:0]		nn;

	assign nn = { rom_addr[3:2], rom_addr[1:0] + x[1:0] };
	
	always @ ( nn ) begin
		case (nn)
			// 1 vilasos, 3 sotet
			 0: rom_data = 0;
			 1: rom_data = 0;
			 2: rom_data = 0;
			 3: rom_data = 0;
			// 2 vilagos, 2 sotet
			 4: rom_data = 0;
			 5: rom_data = 1;
			 6: rom_data = 0;
			 7: rom_data = 1;
			// 3 vilagos, 1 sotet
			 8: rom_data = 0;
			 9: rom_data = 1;
			10: rom_data = 1;
			11: rom_data = 1;
			// 4 vilagos, 0 sotet
			12: rom_data = 1;
			13: rom_data = 1;
			14: rom_data = 1;
			15: rom_data = 1;
		endcase
	end
endmodule

module rom2( rom_addr, x, rom_data );
	input 	[3:0]		rom_addr;		// b3 b2 b1 b0
												//	 ---	^	 ^
												//	  |	|	 |
												//	  |	|	  -- lcd_x_ct
												//	  |	 ------ lcd_y_ct
												//	  |
												//		---------- bit_grayscale

	input		[1:0]		x;					// frame_ct

	output				rom_data;		// kimeneti ertek
	reg					rom_data;
	wire		[3:0]		nn;

	assign nn = { rom_addr[3:2], rom_addr[1:0] + x[1:0] };
	
	always @ ( nn ) begin
		case (nn)
			// 1 vilasos, 3 sotet
			 0: rom_data = 0;
			 1: rom_data = 0;
			 2: rom_data = 0;
			 3: rom_data = 0;
			// 2 vilagos, 2 sotet
			 4: rom_data = 1;
			 5: rom_data = 0;
			 6: rom_data = 1;
			 7: rom_data = 0;
			// 3 vilagos, 1 sotet
			 8: rom_data = 1;
			 9: rom_data = 1;
			10: rom_data = 1;
			11: rom_data = 0;
			// 4 vilagos, 0 sotet
			12: rom_data = 1;
			13: rom_data = 1;
			14: rom_data = 1;
			15: rom_data = 1;
		endcase
	end
endmodule

module rom1( rom_addr, x, rom_data );
	input 	[3:0]		rom_addr;		// b3 b2 b1 b0
												//	 ---	^	 ^
												//	  |	|	 |
												//	  |	|	  -- lcd_x_ct
												//	  |	 ------ lcd_y_ct
												//	  |
												//		---------- bit_grayscale

	input		[1:0]		x;					// frame_ct

	output				rom_data;		// kimeneti ertek
	reg					rom_data;
	wire		[3:0]		nn;

	assign nn = { rom_addr[3:2], rom_addr[1:0] + x[1:0] };
	
	always @ ( nn ) begin
		case (nn)
			// 1 vilasos, 3 sotet
			 0: rom_data = 0;
			 1: rom_data = 0;
			 2: rom_data = 0;
			 3: rom_data = 0;
			// 2 vilagos, 2 sotet
			 4: rom_data = 0;
			 5: rom_data = 1;
			 6: rom_data = 0;
			 7: rom_data = 1;
			// 3 vilagos, 1 sotet
			 8: rom_data = 1;
			 9: rom_data = 1;
			10: rom_data = 0;
			11: rom_data = 1;
			// 4 vilagos, 0 sotet
			12: rom_data = 1;
			13: rom_data = 1;
			14: rom_data = 1;
			15: rom_data = 1;
		endcase
	end
endmodule

module rom0( rom_addr, x, rom_data );
	input 	[3:0]		rom_addr;		// b3 b2 b1 b0
												//	 ---	^	 ^
												//	  |	|	 |
												//	  |	|	  -- lcd_x_ct
												//	  |	 ------ lcd_y_ct
												//	  |
												//		---------- bit_grayscale

	input		[1:0]		x;					// frame_ct

	output				rom_data;		// kimeneti ertek
	reg					rom_data;
	wire		[3:0]		nn;

	assign nn = { rom_addr[3:2], rom_addr[1:0] + x[1:0] };
	
	always @ ( nn ) begin
		case (nn)
			// 1 vilasos, 3 sotet
			 0: rom_data = 0;
			 1: rom_data = 0;
			 2: rom_data = 0;
			 3: rom_data = 0;
			// 2 vilagos, 2 sotet
			 4: rom_data = 1;
			 5: rom_data = 0;
			 6: rom_data = 1;
			 7: rom_data = 0;
			// 3 vilagos, 1 sotet
			 8: rom_data = 1;
			 9: rom_data = 0;
			10: rom_data = 1;
			11: rom_data = 1;
			// 4 vilagos, 0 sotet
			12: rom_data = 1;
			13: rom_data = 1;
			14: rom_data = 1;
			15: rom_data = 1;
		endcase
	end
endmodule


/********************************************************************\
*	Priorita encoder a RAM hozzaferesek engedelyezesehez	            *  			 
\********************************************************************/
 
module mem_prioritas_encoder(
	input_memreq , 	//  8-bit input 
	out, 					//  5 bit output
	out_layer_num		//	 2 bit output
  	);
  
  	output [4:0] out ;
  	reg 	 [4:0] out ;
	output [1:0] out_layer_num;
	reg	 [1:0] out_layer_num;
  	input  [7:0] input_memreq ; 
	
	always @ ( input_memreq ) begin 
		if ( input_memreq[0] == 1 ) 		out = 1;  // CPU olvasas
		else if ( input_memreq[1] == 1 ) out = 4;  // CPU iras
		else if ( input_memreq[2] == 1 ) begin out = 7 ; out_layer_num = 0; end  // layer 1 olvasas ( 1bit/pixel )
		else if ( input_memreq[3] == 1 ) begin out = 10; out_layer_num = 1; end  // layer 2 olvasas ( 1bit/pixel )
		else if ( input_memreq[4] == 1 ) begin out = 13; out_layer_num = 2; end  // layer 3 olvasas ( 1bit/pixel )
		else if ( input_memreq[5] == 1 ) begin out = 16; out_layer_num = 3; end  // layer 4 olvasas ( 1bit/pixel )
		else if ( input_memreq[6] == 1 ) out = 19; // layer 5 olvasas ( 2bit/pixel )
		else if ( input_memreq[7] == 1 ) out = 22; // layer 6 olvasas ( karakteres )
		else out = 0;
	end
endmodule 




